"""
These actually populate assignments in the
canvas sandbox with test data

Created by adam on 1/22/20
"""
import canvasapi

__author__ = 'adam'

if __name__ == '__main__':
    pass

from CanvasHacks.Models.users import StudentUser
from faker import Faker

fake = Faker()

# ------------------------------ Survey answer tools
import random

def answer_essay_survey_questions( questions):
    # make answers
    answers = []
    for q in questions:
        answers.append({
            'id' : q.id,
            'answer': "\n".join([fake.paragraph() for _ in range(0,5)])
        })
    return answers

def answer_multiple_choice_survey_questions(questions):
    answers = []
    for q in questions:
        choices = q.answers
        random.shuffle(choices)
        answers.append({
            'id': q.id,
            'answer': choices[0]['id']
        })
    return answers


def answer_survey(student_token, course_id, quiz_id, essay_questions, multiple_choice):
    student = StudentUser(student_token, course_id, quiz_id=quiz_id)
    quiz = student.course.get_quiz(quiz_id)

    # Create a submission object
    try:
        submission = quiz.create_submission()
    except canvasapi.exceptions.Conflict:
        # a submission was already created
        # so get that one
        submission = [s for s in quiz.get_submissions()][0]

    # make answers
    answers = []
    answers += answer_essay_survey_questions(essay_questions)
    answers += answer_multiple_choice_survey_questions(multiple_choice)

    # submit answers
    question_subs = submission.answer_submission_questions(quiz_questions=answers)

    # Complete the quiz
    submission.complete()
    print("Answers created for user {}".format(submission.user_id))

    # return for use in testing
    return answers



# ------------------------------- Discussion answer tools

def populate_discussion( tokens: list, topic_id: int, deep: int, course_id: int ):
    """Uploads test data to a discussion forum
    :param topic_id: The identifier of the canvas discussion to populate
    :param deep: The number of mutual replies to each post
    :param course_id: The canvas id of the course
    :param tokens: List of student access token strings
    """
    posts = [ ]
    student_users = [ ]
    for t in tokens:
        student_users.append( StudentUser( t, course_id, topic_id=topic_id ) )

    for su in student_users:
        msg = fake.paragraph()
        e = su.post_entry( msg )
        posts.append( e )
    for i in range( 0, deep ):
        for s1 in student_users:
            for entry in s1.discussion_entries:
                # every other student comments on each entry
                for s2 in student_users:
                    msg = fake.paragraph()
                    e = s2.discussion.get_entries( [ entry.id ] )
                    r = e[ 0 ].post_reply( message=msg )
                    posts.append( r )


# ---------------------- Assignment answer tools

def populate_assignment( tokens: list, assignment_id: int, course_id: int, num_paragraphs=1 ):
    """Uploads a paragraph of response text for each token provided
    in tokens.
    :param assignment_id: The identifier of the canvas unit to populate
    :param course_id: The canvas id of the course
    :param tokens: List of student access token strings
    :returns List of Submission objects returned from server
    """
    submissions = [ ]

    for t in tokens:
        msg = '\n '.join([fake.paragraph() for _ in range(0,num_paragraphs)])
        sub = {
            'submission_type': 'online_text_entry',
            'body': msg,
        }
        s1 = StudentUser( t, course_id, assignment_id=assignment_id )
        result = s1.assignment.submit( sub )
        submissions.append( result )
    return submissions


# -------------------- Quiz answer tools
def get_question_ids( course, quiz_id ):
    """Returns list of ids for questions comprising quiz"""
    qids = [ q.id for q in course.get_quiz( quiz_id ).get_questions() ]
    print( "{} question ids retrieved for quiz {}".format( len( qids ), quiz_id ) )
    return qids


def answer_quiz_for_student( student_token, course_id, quiz_id, question_ids ):
    """Answers all quiz questions for one student"""
    student = StudentUser( student_token, course_id, quiz_id=quiz_id )
    quiz = student.course.get_quiz( quiz_id )

    # Create a submission object
    try:
        submission = quiz.create_submission()
    except canvasapi.exceptions.Conflict:
        # a submission was already created
        # so get that one
        submission = [ s for s in quiz.get_submissions() ][ 0 ]

    # make answers
    answers = [ ]
    for qid in question_ids:
        answers.append( {
            'id': qid,
            'answer': fake.paragraph()
        } )

    # submit answers
    question_subs = submission.answer_submission_questions( quiz_questions=answers )

    # Complete the quiz
    submission.complete()
    print( "Answers created for user {}".format( submission.user_id ) )

    # return quizSubmissionQuestions for use in testing
    # answer text will be stored on answer attribute
    return question_subs


def populate_quiz_w_text_answers( course, quiz_id, student_tokens ):
    """
    Answers all text entry questions for a quiz for all students

    Returns a list of QuizSubmissionQuestions which contains the answer text
    in answer attribute"""
    answers = [ ]
    question_ids = get_question_ids( course, quiz_id )
    for t in student_tokens:
        r = answer_quiz_for_student( t, course.id, quiz_id, question_ids )
        answers.append( r )
    return answers



"""
Created by adam on 3/9/20
"""
__author__ = 'adam'

from unittest.mock import MagicMock, create_autospec

from CanvasHacks.GradingMethods.base import IGradingMethod
from CanvasHacks.GradingCorrections.penalities import IPenalizer
from CanvasHacks.GradingHandlers.quiz import QuizGrader
from CanvasHacks.Repositories.interfaces import ISubmissionRepo
from TestingBase import TestingBase

from factories.RepositoryMocks import ContentRepositoryMock

if __name__ == '__main__':
    pass


class TestQuizGrader( TestingBase ):

    def setUp(self):
        self.config_for_test()
        self.points_per_question = self.fake.random.randint(1,100)

        self.create_new_and_preexisting_students()

        self.penalizer = create_autospec(IPenalizer)
        self.grade_method = create_autospec(IGradingMethod)

        self.activity = MagicMock(grade_method=self.grade_method,
                                  penalizer=self.penalizer)

        self.work_repo = ContentRepositoryMock( activity=self.activity,
                                                points_per_question=self.points_per_question )

        self.work_repo.create_quiz_repo_data(self.student_ids,
                                             submitted_at=self.fake.date_time_this_century(),
                                             make_dataframe=True)

        self.submission_repo = create_autospec(ISubmissionRepo)

        self.obj = QuizGrader(self.work_repo, self.submission_repo)


    def test__get_score( self ):
        content = self.fake.paragraph()
        # call
        self.obj._get_score(content)
        # check
        self.grade_method.grade.assert_called()
        self.grade_method.grade.assert_called_with(content)

    def test__get_score_custom_on_empty( self ):
        # Custom on empty case
        self.grade_method.grade = MagicMock( return_value=None )
        content = self.fake.paragraph()
        on_empty = 'taco'

        # call
        result = self.obj._get_score( content, on_empty=on_empty )

        # check
        self.assertEqual(on_empty, result, "returns custom value when empty")
        self.grade_method.grade.assert_called()
        self.grade_method.grade.assert_called_with( content )

    def test__grade_row_no_penalty( self ):
        fudge_points = 0
        self.penalizer.get_fudge_points = MagicMock(return_value=fudge_points)
        self.grade_method.grade = MagicMock( return_value=self.points_per_question )
        row = self.work_repo.data.iloc[0]

        # call
        result = self.obj._grade_row(row)

        # check things called with expected values
        self.grade_method.grade.assert_called()
        for qid, column_name in self.work_repo.question_columns:
            # grade method called on each question field in row
            self.grade_method.grade.assert_any_call(row[column_name])

        # check fudge points
        result_fudge_points = result[ 'data' ][ "quiz_submissions" ][0]['fudge_points']
        self.assertEqual(result_fudge_points, 0, "zero fudge points returned")

        # check question scores
        result_question_dict = result[ 'data' ][ "quiz_submissions" ][ 0 ][ 'questions' ]
        for qid, cname in self.work_repo.question_columns:
            self.assertIn(qid, result_question_dict, "expected dictionary key")
            self.assertIn('score', result_question_dict[qid], "expected internal dict key")
            self.assertEqual(result_question_dict[qid]['score'], self.points_per_question, "expected question score")

    def test_grade_row_penalty( self ):
        fudge_points = -50
        self.penalizer.get_fudge_points = MagicMock(return_value=fudge_points)
        self.grade_method.grade = MagicMock( return_value=self.points_per_question )
        row = self.work_repo.data.iloc[0]

        # call
        result = self.obj._grade_row(row)

        # check things called with expected values
        self.grade_method.grade.assert_called()
        for qid, column_name in self.work_repo.question_columns:
            # grade method called on each question field in row
            self.grade_method.grade.assert_any_call(row[column_name])

        # check fudge points
        result_fudge_points = result[ 'data' ][ "quiz_submissions" ][0]['fudge_points']
        self.assertEqual(result_fudge_points, fudge_points, "expected fudge points returned")

        # check question scores
        result_question_dict = result[ 'data' ][ "quiz_submissions" ][ 0 ][ 'questions' ]
        for qid, cname in self.work_repo.question_columns:
            self.assertIn(qid, result_question_dict, "expected dictionary key")
            self.assertIn('score', result_question_dict[qid], "expected internal dict key")
            self.assertEqual(result_question_dict[qid]['score'], self.points_per_question, "expected question score")


    def test__penalty_message( self ):
        self.skipTest('todo')


class TestQuizGraderFunctionalTests(TestingBase):

    def test_grade( self ):
        self.skipTest('todo')
        # todo Mixed credit and no credit students
        # todo Different total scores
        # todo Different attempts



"""
Created by adam on 1/5/20
"""
import pickle

__author__ = 'adam'

import os
import flask
import requests

import google.oauth2.credentials
import google_auth_oauthlib.flow
import googleapiclient.discovery


ROOT = os.getenv( "HOME" )

CREDS_DIR = "{}/Dropbox/CanvasTools/private/google".format(ROOT)
MATERIALS_DIR = """{}/Box\ Sync/TEACHING/Phil\ 305\ Business\ ethics/public-files""".format(ROOT)

# Credentials after oath is complete stored here
CRED_FILEPATH = '{}/credentials.dat'.format(CREDS_DIR)

SCOPES = ['https://www.googleapis.com/auth/blogger', ]
blog_id = 4074236150842536786

body_data = {
    "kind": "blogger#post",
    "blog": {
        "id": blog_id
    },
    "title": "A new post",
    "content": "With <b>exciting</b> content..."
}

# This variable specifies the name of a file that contains the OAuth 2.0
# information for this application, including its client_id and client_secret.
CLIENT_SECRETS_FILE = "{}/client_secret.json".format(CREDS_DIR)

# This OAuth 2.0 access scope allows for full read/write access to the
# authenticated user's account and requires requests to use an SSL connection.
# SCOPES = ['https://www.googleapis.com/auth/drive.metadata.readonly']
API_SERVICE_NAME = 'blogger'
API_VERSION = 'v3'

app = flask.Flask(__name__)
# Note: A secret key is included in the sample so that it works.
# If you use this code in your application, replace this with a truly secret
# key. See http://flask.pocoo.org/docs/0.12/quickstart/#sessions.
app.secret_key = 'REPLACE ME - this value is here as a placeholder.'


@app.route('/')
def index():
    return print_index_table()


@app.route('/test')
def test_api_request():
    if 'credentials' not in flask.session:
        return flask.redirect('authorize')

    # Load credentials from the session.
    credentials = google.oauth2.credentials.Credentials(
        **flask.session['credentials'])

    service = googleapiclient.discovery.build(
        API_SERVICE_NAME, API_VERSION, credentials=credentials)

    b = service.posts()
    result = b.insert(blogId=blog_id, body=body_data).execute()
    # files = drive.files().list().execute()

    # Save credentials back to session in case access token was refreshed.
    # ACTION ITEM: In a production app, you likely want to save these
    #              credentials in a persistent database instead.
    flask.session['credentials'] = credentials_to_dict(credentials)

    return flask.jsonify(**result)


@app.route('/authorize')
def authorize():
    # Create flow instance to manage the OAuth 2.0 Authorization Grant Flow steps.
    flow = google_auth_oauthlib.flow.Flow.from_client_secrets_file(
        CLIENT_SECRETS_FILE, scopes=SCOPES)

    # The URI created here must exactly match one of the authorized redirect URIs
    # for the OAuth 2.0 client, which you configured in the API Console. If this
    # value doesn't match an authorized URI, you will get a 'redirect_uri_mismatch'
    # error.
    flow.redirect_uri = flask.url_for('oauth2callback', _external=True)

    authorization_url, state = flow.authorization_url(
        # Enable offline access so that you can refresh an access token without
        # re-prompting the user for permission. Recommended for web server apps.
        access_type='offline',
        # Enable incremental authorization. Recommended as a best practice.
        include_granted_scopes='true')

    # Store the state so the callback can verify the auth server response.
    flask.session['state'] = state

    return flask.redirect(authorization_url)


@app.route('/oauth2callback')
def oauth2callback():
    # Specify the state when creating the flow in the callback so that it can
    # verified in the authorization server response.
    state = flask.session['state']

    flow = google_auth_oauthlib.flow.Flow.from_client_secrets_file(
        CLIENT_SECRETS_FILE, scopes=SCOPES, state=state)
    flow.redirect_uri = flask.url_for('oauth2callback', _external=True)

    # Use the authorization server's response to fetch the OAuth 2.0 tokens.
    authorization_response = flask.request.url
    flow.fetch_token(authorization_response=authorization_response)

    # Store credentials in the session.
    # ACTION ITEM: In a production app, you likely want to save these
    #              credentials in a persistent database instead.
    credentials = flow.credentials

    # todo store credentials
    # # we check if the file to store the credentials exists
    # if not os.path.exists(CRED_FILEPATH):
    #     with open(CRED_FILEPATH, 'wb') as credentials_dat:
    #     pickle.dump(credentials, credentials_dat)
    # else:
    #     with open(CRED_FILEPATH, 'rb') as credentials_dat:
    #     credentials = pickle.load(credentials_dat)
    #
    # if credentials.expired:
    #     credentials.refresh(Request())

    flask.session['credentials'] = credentials_to_dict(credentials)

    return flask.redirect(flask.url_for('test_api_request'))


# @app.route('/revoke')
# def revoke():
#     if 'credentials' not in flask.session:
#         return ('You need to <a href="/authorize">authorize</a> before ' +
#                 'testing the code to revoke credentials.')
#
#     credentials = google.oauth2.credentials.Credentials(
#         **flask.session['credentials'])
#
#     revoke = requests.post(
#         'https://accounts.google.com/o/oauth2/revoke',
#         params={'token': credentials.token},
#         headers={'content-type': 'application/x-www-form-urlencoded'})
#
#     status_code = getattr(revoke, 'status_code')
#     if status_code == 200:
#         return ('Credentials successfully revoked.' + print_index_table())
#     else:
#         return ('An error occurred.' + print_index_table())
#
#
# @app.route('/clear')
# def clear_credentials():
#     if 'credentials' in flask.session:
#         del flask.session['credentials']
#     return ('Credentials have been cleared.<br><br>' + print_index_table())


def credentials_to_dict(credentials):
    return {
        'token': credentials.token,
        'refresh_token': credentials.refresh_token,
        'token_uri': credentials.token_uri,
        'client_id': credentials.client_id,
        'client_secret': credentials.client_secret,
        'scopes': credentials.scopes
    }


def print_index_table():
    return (
            '<table>' + '<tr><td><a href="/test">Test an API request</a></td>' +
            '<td>Submit an API request and see a formatted JSON response. ' +
            '    Go through the authorization flow if there are no stored ' +
            '    credentials for the user.</td></tr>' +
            '<tr><td><a href="/authorize">Test the auth flow directly</a></td>' +
            '<td>Go directly to the authorization flow. If there are stored ' +
            '    credentials, you still might not be prompted to reauthorize ' +
            '    the application.</td></tr>' +
            '<tr><td><a href="/revoke">Revoke current credentials</a></td>' +
            '<td>Revoke the access token associated with the current user ' +
            '    session. After revoking credentials, if you go to the test ' +
            '    page, you should see an <code>invalid_grant</code> error.' +
            '</td></tr>' +
            '<tr><td><a href="/clear">Clear Flask session credentials</a></td>' +
            '<td>Clear the access token currently stored in the user session. ' +
            '    After clearing the token, if you <a href="/test">test the ' +
            '    API request</a> again, you should go back to the auth flow.' +
            '</td></tr></table>')


if __name__ == '__main__':
    # When running locally, disable OAuthlib's HTTPs verification.
    # ACTION ITEM for developers:
    #     When running in production *do not* leave this option enabled.
    os.environ['OAUTHLIB_INSECURE_TRANSPORT'] = '1'

    # Specify a hostname and port that are set as a valid redirect URI
    # for your API project in the Google API Console.
    app.run('localhost', 8084, debug=True)



"""
Only used to influence variables in environoment upon
first load during testing.

Created by adam on 2/24/20
"""
__author__ = 'adam'

TEST = False
"""Whether environment should load into test state"""

use_api = True
"""Whether to interact with the server during testing"""

TEST_WITH_FILE_DB = False
"""Whether to connect to a file based db. If false, we use an in-memory db"""


if __name__ == '__main__':
    pass


"""
Created by adam on 2/24/20
"""
import unittest
from tests.TestingBase import TestingBase

from faker import Faker

from CanvasHacks.DAOs.sqlite_dao import SqliteDAO
from CanvasHacks.Models.status_record import StatusRecord
from CanvasHacks.Repositories.status import StatusRepository
from tests.factories.ModelFactories import student_factory
from tests.factories.PeerReviewedFactories import unit_factory
fake = Faker()


class TestStatusRepository( TestingBase ):

    def setUp( self ):
        self.config_for_test()
        self.dao = SqliteDAO()
        print( "Connected to testing db" )
        self.session = self.dao.session

        self.student = student_factory()
        self.unit = unit_factory()
        self.activity = fake.random.choice( self.unit.components )

        self.obj = StatusRepository( self.dao, self.activity )

    def _create_record( self ):
        rec = StatusRecord( student_id=self.student.student_id, activity_id=self.activity.id )
        self.session.add( rec )
        self.session.commit()
        return rec

    def test_get_or_create_record( self ):
        self.skipTest('todo')

    def test_get_record( self ):
        self.skipTest('todo')

    def test_create_record( self ):
        r = self.obj.create_record( self.student )

        # check
        self.assertIsInstance( r, StatusRecord )
        self.assertEqual( r.student_id, self.student.student_id )
        self.assertEqual( r.activity_id, self.activity.id )
        self.assertTrue( r.submitted is None, "Submission is empty" )
        self.assertTrue( r.notified is None, "Notification is empty" )
        self.assertTrue( r.results is None, "Sent results is empty" )

    def test_record_notified( self ):
        self.skipTest('todo')

    def test_record_submitted( self ):
        self.skipTest('todo')


# class TestComplexStatusRepository( TestingBase ):
#
#     def setUp( self ):
#         self.config_for_test()
#         self.dao = SqliteDAO()
#         print( "Connected to testing db" )
#         self.session = self.dao.session
#
#         self.activity_data = activity_data_factory()
#         self.target_student = student_factory()
#         self.reviewing_student = student_factory()
#         self.reviewed_student = student_factory()
#         self.activity_id = fake.random.randint( 1111, 99999 )
#         self.unit = unit_factory()
#
#         self.obj = ComplexStatusRepository( MagicMock(), self.unit )
#
#     def test__handle_id( self ):
#         self.fail()
#
#     def test_load( self ):
#         self.fail()
#
#     def test_get_unit_records( self ):
#         self.fail()
#
#     def test_get_student( self ):
#         self.fail()
#
#     def test_create_record( self ):
#         self.fail()
#
#     def test_record_review_assignments_both_records_exist( self ):
#         self.obj.get_student_record = MagicMock( return_value=self.reviewing_student )
#
#         self.obj.record_review_assignments( (self.reviewing_student.student_id, self.reviewing_student.student_id) )
#
#     def test_record_review_assignments_reviewer_record_exists( self ):
#         self.fail()
#
#     def test_record_peer_review_results_sent( self ):
#         self.fail()
#
#     def test_record_metareview_results_sent( self ):
#         self.fail()
#


if __name__ == '__main__':
    unittest.main()



"""
Tools for downloading data from assignments that do not
use quiz reports
Created by adam on 2/26/20
"""
from CanvasHacks.Loaders.interfaces import ILoader
from CanvasHacks.Files.QuizReportFileTools import save_downloaded_report

__author__ = 'adam'

from CanvasHacks.Repositories.submissions import AssignmentSubmissionRepository

if __name__ == '__main__':
    pass


class AssignmentDownloadLoader( ILoader ):
    """Loads all records for quiz"""

    @staticmethod
    def load( activity, course, save=True, **kwargs ):
        """
        Handles loading data for a non-quiz unit
        :param activity:
        :param course:
        :param kwargs:
        :return: DataFrame
        """
        # We need the canvas.api.unit object
        assignment = AssignmentDownloadLoader.get_assignment( course, activity )
        subRepo = AssignmentSubmissionRepository( assignment )

        # parse out already graded submissions
        # subRepo.data =[j for j in subRepo.data if j[0].grade != 'complete']
        if save:
            # Want to have all the reports be formatted the same
            # regardless of whether we manually or programmatically
            # downloaded them. Thus we save before doing anything to them.
            save_downloaded_report( activity, subRepo.frame )

        return subRepo.frame



"""
Created by adam on 3/22/20
"""
__author__ = 'adam'

from sqlalchemy import Column, Integer
from sqlalchemy.dialects.sqlite import DATETIME

from CanvasHacks import environment as env
# from sqlalchemy.ext.declarative import declarative_base
from CanvasHacks.DAOs.sqlite_dao import Base
from CanvasHacks.Models.model import Model
from CanvasHacks.TimeTools import current_utc_timestamp

if __name__ == '__main__':
    pass


class SubmissionRecord( Base, Model ):
    """
    Record of a student completing an assignment.
    The first time the student completed the assignment.

    Created in CAN-61

    """
    __tablename__ = env.SUBMISSION_TABLE_NAME

    activity_id = Column(Integer, primary_key=True, nullable=False)
    """Id of the activity in which the feedback was created"""

    student_id = Column( Integer, primary_key=True, nullable=False )
    """ The student whom this record concerns """

    submitted_at = Column(DATETIME, nullable=True)
    """ Timestamp of when the feedback was sent to them"""



"""
Created by adam on 3/1/20
"""
__author__ = 'adam'

import pandas as pd
from unittest.mock import patch, MagicMock
from CanvasHacks.Definitions.activity import Activity
from CanvasHacks.Definitions.unit import Unit
from CanvasHacks.PeerReviewed.SkaaMaker import UnitDefinitionsLoader
from tests.TestingBase import TestingBase
import CanvasHacks.environment as env
from faker import Faker
fake = Faker()
import pytz

if __name__ == '__main__':
    pass


class TestUnitDefinitionsLoader( TestingBase ):

    def setUp(self):
        self.test_datafile = "{}/Unit details.xlsx".format( env.TEST_DATA_PATH)
        self.obj = UnitDefinitionsLoader()
        self.obj.load( self.test_datafile )

    def test_load( self ):
        self.assertIsInstance( self.obj.unit_details, pd.DataFrame)
        self.assertIsInstance( self.obj.start_dates, pd.DataFrame)

    def test_make_unit( self ):
        unit_number = 3
        start = pd.to_datetime(fake.date_time_this_century( tzinfo=pytz.utc ))
        u = self.obj.make_unit(unit_number, start)

        # Check
        self.assertIsInstance(u, Unit)
        self.assertEqual(u.unit_number, unit_number)

    def test_create_units( self ):
        self.obj.create_units()

        self.assertTrue(len(self.obj.units) > 0)
        self.assertTrue(len(self.obj.units) == 5)

        for unit in self.obj.units:
            for tp in unit.components:
                self.assertIsInstance(tp, Activity)

    @patch('CanvasHacks.PeerReviewed.SkaaMaker.UnitDefinitionsLoader.CreationHandlerFactory')
    def test_run( self, makerMock ):
        self.obj.create_units()
        course = MagicMock()

        self.run(course)

        course.assert_called()
    #     obj = UnitDefinitionsLoader()
    #     obj.load( self.test_datafile )
    #
    #     obj.run()




"""
Functions and tools for making requests to the
canvas api specific to the peer reviewded assignments

Created by adam on 12/24/19
"""
from CanvasHacks.Models.student import Student
from CanvasHacks.PeerReviewed.Submissions import Submission, ReviewSubmission

__author__ = 'adam'

if __name__ == '__main__':
    pass


def get_all_submissions( course_id, quiz_id, per_page=42 ):
    """Downloads all initial submissions for the unit"""
    # todo: query to server goes here
    # Holds the results from the request to server
    results = [ ]
    # Submission objects to return
    subs = [ ]

    for r in results:
        # todo: check correct field names
        student = Student( r[ 'id' ], r[ 'name' ] )


def get_all_peer_reviews( review_activity, **kwargs ):
    """Returns objects with the format:
        {
      // The assessors user id
      "assessor_id": 23,
      // The id for the asset associated with this Peer Review
      "asset_id": 13,
      // The type of the asset
      "asset_type": "Submission",
      // The id of the Peer Review
      "id": 1,
      // The user id for the owner of the asset
      "user_id": 7,
      // The state of the Peer Review, either 'assigned' or 'completed'
      "workflow_state": "assigned",
      // the User object for the owner of the asset if the user include parameter is
      // provided (see user API) (optional)
      "user": "User",
      // The User object for the assessor if the user include parameter is provided
      // (see user API) (optional)
      "assessor": "User",
      // The submission comments associated with this Peer Review if the
      // submission_comment include parameter is provided (see submissions API)
      // (optional)
      "submission_comments": "SubmissionComment"
    }
    """
    # query happens
    # todo make query happen. Include 'user' parameter and 'submission_comment' param
    results = [ ]
    # Submission objects to return
    subs = [ ]

    for r in results:
        # todo: check correct field names
        submitter = Student( r[ 'user_id' ], **r[ 'user' ] )
        reviewer = Student( r[ 'assessor_id' ], **r[ 'assessor' ] )
        submission = ReviewSubmission( submitter, review_activity, **r )
        subs.append( submission )


def get_assignment_data_for_student( assignment, student ):
    pass




"""
Created by adam on 2/1/19
"""
from typing import List
# from AdamTools.Text.TextProcessingTools import WordFreq
from CanvasHacks.Text.stats import WordFreq

__author__ = 'adam'


class DataStoreNew(object):
    """All downloaded and score data gets stored in this object
    Methods using the data will expect an instance.
    New as of CAN-24
    """
    credit: List[ int ]

    def __init__(self, activity):
        self.activity = activity
        self.course_id = activity.course_id
        self.assignment_name = activity.name
        self.assignment_id = activity.id
        # New (can-24) thing to hold the scores
        # should contain tuples (submission, pct credit)
        self.results = []

    @property
    def credit( self ):
        """Alias so can use interfaces for old version"""
        return self.results

    @property
    def submissions( self ):
        """Alias so can use interfaces for old version"""
        r = [s[0].attributes for s in self.results]
        for s in r:
            s['student_id'] = s['user_id']
        return r


    def print_counts( self ):
        m = "Tentatively assigning credit to {} submissions; no credit to {} submissions."
        print(m.format(len(self.credit), len(self.no_credit)))


class DataStore(object):
    """All downloaded and score data gets stored in this object
    Methods using the data will expect an instance.
    """
    credit: List[ int ]

    def __init__(self, assignment_id=None,  assignment_name=None, course_id=None):
        self.course_id = course_id
        self.assignment_name = assignment_name
        self.assignment_id = assignment_id
        # New (can-24) thing to hold the scores
        # should contain tuples (submission, pct credit)
        self.scores = []
        # List of ids from students receiving credit
        self.credit = []
        self.no_credit = []
        self.submissions = []

    def print_counts( self ):
        m = "Tentatively assigning credit to {} submissions; no credit to {} submissions."
        print(m.format(len(self.credit), len(self.no_credit)))

    # @property
    # def ids_receiving_credit( self):
    #     # if assignment_id is not None:
    #     #     c = list(filter(lambda x: x['unit'] == assignment_id, self.credit))[0]
    #     #     return c['ids']
    #     return self.credit[0].ids


class BagStore(object):

    def __init__(self):
        # Preserves the distinct submissions from students
        self.assignment_bags = {}
        # Holds 1 bag per unit containing the contents of
        # all student bags
        self.assignment_words = {}

    @property
    def assignment_names(self):
        return [name for name in self.assignment_bags.keys()]

    @property
    def unique_words(self):
        return list(set(self.all_words))

    @property
    def all_bags(self):
        """Returns a list containing lists of words
        Each of the word list is a student's submission,
        but all the assignments are combined
        """
        return []

    @property
    def all_words(self):
        """Returns a list of all the words
        submitted on all the assignments."""
        a = []
        for name in self.assignment_bags.keys():
            for bag in self.assignment_bags[name]:
                a += bag
        return a

    def add_assignment_bags(self, name, bags):
        """bags is a list of lists. Each list should
        contain words from 1 student's submission on the unit"""
        self.assignment_bags[name] = bags
        self.assignment_words[name] = []
        for b in bags:
            self.assignment_words[name] += [w for w in b]

    def get_assignment_frequencies(self, name):
        return WordFreq(self.assignment_words[name])


if __name__ == '__main__':
    pass


"""
Created by adam on 12/26/19
"""
from unittest import TestCase

from CanvasHacks.PeerReviewed.Submissions import *
from tests.factories.CanvasApiFactories import *
from tests.factories.ModelFactories import *
from tests.factories.PeerReviewedFactories import *

__author__ = 'adam'

if __name__ == '__main__':
    pass

# class TestSubmission( TestCase ):
#     def test_on_time( self ):
#         self.fail()


class TestInitialSubmission( TestCase ):
    def setUp( self ):
        self.test_data = activity_data_factory()
        self.activity = InitialWork( **self.test_data )
        self.student = student_factory()
        self.query_result = submission_result_factory( self.activity, self.student )

    def test_on_time( self ):
        obj = InitialSubmission( self.activity, self.query_result )
        self.assertTrue( obj.on_time, "Returns true when on time" )

        query_result = submission_result_factory( self.activity, self.student, on_time=False )
        obj = InitialSubmission( self.activity, query_result )
        self.assertFalse( obj.on_time, "Returns false when late" )

    def test_submitter_points( self ):
        obj = InitialSubmission( self.activity, self.query_result )
        self.assertEqual( obj.submitter_points, self.activity.completion_points,
                          "Assigns completion points when on time" )

    def test_process( self ):
        obj = InitialSubmission( self.activity, self.query_result )
        self.assertTrue( obj.submitter == self.student, "Submitting student sets properly" )
        self.assertEqual( obj.completion_date, self.query_result[ 'submitted_at' ], "completino date sets properly" )


class TestReviewSubmission( TestCase ):

    def setUp( self ):
        self.reviewer = student_factory()
        self.student = student_factory()
        self.test_data = activity_data_factory()
        self.activity = Review( **self.test_data )
        self.query_result = peer_review_result_factory( self.student, self.reviewer )

    def test_process( self ):
        obj = ReviewSubmission( self.activity, self.query_result )
        self.assertTrue( obj.submitter == self.student, "Submitting student sets properly" )
        self.assertTrue( obj.reviewer == self.reviewer, "Reviewing student sets properly" )
        # self.assertEqual(obj.completion_date, self.query_result['submitted_at'], "completino date sets properly")

    def test_is_complete( self ):
        # self.query_result['workflow_state'] = 'completed'
        obj = ReviewSubmission( self.activity, self.query_result )
        self.assertTrue( obj.is_complete, "Returns true when workflow state is completed" )

        query_result = peer_review_result_factory( self.student, self.reviewer, completed=False )
        # self.query_result['workflow_state'] = 'assigned'
        obj = ReviewSubmission( self.activity, query_result )
        self.assertFalse( obj.is_complete, "Returns false when workflow state is not completed" )

    def test_reviewer_points( self ):
        obj = ReviewSubmission( self.activity, self.query_result )
        self.assertEqual( obj.reviewer_points, self.activity.completion_points,
                          "Assigns completion points when the review is complete" )

        query_result = peer_review_result_factory( self.student, self.reviewer, completed=False )
        obj = ReviewSubmission( self.activity, query_result )
        self.assertEqual( obj.reviewer_points, 0, "Assigns 0 when the review is incomplete" )

    def test_submitter_points( self ):
        score = random.randint( 0, 100 )

        obj = ReviewSubmission( self.activity, self.query_result )
        obj.assigned_score = score
        self.assertEqual( obj.submitter_points, score, "Gives assigned score when the reviewer did their job" )

        query_result = peer_review_result_factory( self.student, self.reviewer, completed=False )
        obj = ReviewSubmission( self.activity, query_result )
        obj.assigned_score = score
        self.assertEqual( obj.submitter_points, self.activity.assignable_points,
                          "Gives max assignable points when the reviewer did not do their job" )



"""
Created by adam on 2/24/20
"""
import pandas as pd

from CanvasHacks.Errors.data_ingestion import NoNewSubmissions, NoStudentWorkDataLoaded, NoWorkDownloaded
from CanvasHacks.Files.FileTools import create_folder
from CanvasHacks.Loaders.interfaces import IAllLoader, INewLoader
from CanvasHacks.Processors.quiz import process_work
from CanvasHacks.Files.QuizReportFileTools import load_activity_data_from_files, load_activity_from_newest_file, load_new,\
    retrieve_quiz_data, save_downloaded_report

__author__ = 'adam'

if __name__ == '__main__':
    pass


# =================================== file

class AllQuizReportFileLoader( IAllLoader ):
    """Loads all records for quiz"""
    failure_message = "Could not load data from file"

    @staticmethod
    def load( activity, course, **kwargs ):
        # course = self.course if course is None else course
        # activity_inviting_to_complete = self.activity_inviting_to_complete if activity_inviting_to_complete is None else activity_inviting_to_complete
        # pass
        student_work_frame = load_activity_data_from_files( activity, course )
        AllQuizReportFileLoader._check_empty( student_work_frame )
        return student_work_frame


class QuizReportFileLoader( IAllLoader ):
    """
    New standard fileloader
    """

    @staticmethod
    def load( activity, course, **kwargs ):
        student_work_frame = load_activity_from_newest_file( activity )
        QuizReportFileLoader._check_empty( student_work_frame )
        return student_work_frame


class NewQuizReportFileLoader( INewLoader ):
    """
    Loads latest data from files

    todo
    """

    @staticmethod
    def load( activity, course, **kwargs ):
        """
        Returns all new records
        :param course:
        :param activity:
        :return: DataFrame
        :raises: NoNewSubmissions
        """
        data = load_new( activity )
        NewQuizReportFileLoader._check_empty( data )
        return data


# ========================= download
class AllQuizReportDownloader( INewLoader ):
    failure_message = "Could not download data."

    @staticmethod
    def load( activity, course, save=True, **kwargs ):
        quiz = AllQuizReportDownloader.get_quiz( course, activity )
        # This will return none if never able to download from a url
        student_work_frame = retrieve_quiz_data( quiz, **kwargs )

        if student_work_frame is None:
            print( 'uhoh' )
            raise NoWorkDownloaded

        if save:
            # Want to have all the reports be formatted the same
            # regardless of whether we manually or programmatically
            # downloaded them. Thus we save before doing anything to them.
            save_downloaded_report( activity, student_work_frame )

        return student_work_frame


class NewQuizReportDownloadLoader( INewLoader ):
    """
    Downloads latest report and returns what's new without
    storing report

    todo
    """

    @staticmethod
    def load( activity, course=None, **kwargs ):
        """
        Returns all new records
        :param course:
        :param activity:
        :return: DataFrame
        :raises: NoNewSubmissions
        """
        # quiz = AllQuizReportDownloader.get_quiz( course, activity_inviting_to_complete )
        # student_work_frame = retrieve_quiz_data( quiz )

        data = load_new( activity )
        NewQuizReportDownloadLoader._check_empty( data )
        return data


class QuizComboLoader( IAllLoader ):
    """
    Attempts to load first by downloading and then falls
    back to loading from a file.
    """

    @staticmethod
    def load( activity, course, save=True, **kwargs ):
        loaders = (AllQuizReportDownloader, QuizReportFileLoader) # AllQuizReportFileLoader)
        for loader in loaders:
            try:
                # The loader should raise the error or return the
                # desired value
                return loader.load( activity, course, save=save, **kwargs )
            except NoWorkDownloaded as e:
                # We ignore so next object will try
                print( e.message )

                # We attempt to create the folder that the file loader will
                # look in because, otherwise, we won't have a handy location
                # to put the downloaded file in the future. (Before, it accidentally happened
                # when we tried to save None instead of a dataframe. That was fixed
                # so now we need to do it explicitly).
                create_folder( activity.folder_path )

            except NoNewSubmissions:
                # We're done, but this may have special handling
                # so we re-raise it
                raise NoNewSubmissions

        # if we made it all the way here, all attempts at loading have failed
        # so we raise this to tell the program to move on
        raise NoStudentWorkDataLoaded


def load_student_work( csv_filepath, submissions ):
    """Loads and processes a csv file containing all student work for the unit
    submissions: DataFrame containing student submission objects
    NB, process_work has been refactored out in CAN-11 but load_student_work
    is still here for legacy uses
    """
    f = pd.read_csv( csv_filepath )
    return process_work( f, submissions )
    # # rename id so will be able to join
    # f.rename( { 'id': 'student_id' }, axis=1, inplace=True )
    # # merge it with matching rows from the submissions frame
    # f = pd.merge( f, submissions, how='left', on=[ 'student_id', 'attempt' ] )
    # f.set_index( 'name', inplace=True )
    # f.sort_index( inplace=True )
    # return f



"""
Created by adam on 10/26/18
"""
__author__ = 'adam'

from CanvasHacks.Files.JournalsFileTools import load_words_to_ignore


def filter_out_terms(bag, exclude=[]):
    """ Applies exclude list and other filters to the wordbag"""
    if len(exclude) == 0:
        exclude = load_words_to_ignore()

    exclude += [ "%s" % i for i in range(0,1000)]
    exclude += ['none']
    #     print(exclude)
    return [w for w in bag if w not in exclude]


# test
# t = ['none', 'a']
# assert(filter_out_terms(t, ['a']) == [])

if __name__ == '__main__':
    pass


"""
Created by adam on 10/1/19
"""
import CanvasHacks.environment as env
from CanvasHacks.Models.student import Student
from CanvasHacks.Repositories.interfaces import IRepo

__author__ = 'adam'

from CanvasHacks.Api.RequestTools import send_multi_page_get_request
from CanvasHacks.Api.UrlTools import make_url


class StudentRepository( IRepo ):
    """Manages information about students downloaded
    using canavasapi
    """

    def __init__( self, course_objects=[] ):
        """
        :param course_objects: List of canvasapi.Course objects or single object
        """
        if not isinstance(course_objects, list):
            course_objects = [course_objects]
        # List of canvas api objects
        self.course_objects = course_objects
        self.data = { }

    def download( self ):
        """Makes request to the server for all students enrolled in the
         course
        Stores student records in self.data with canvas id as keys
        """
        if len(self.course_objects) > 0:
            for course in self.course_objects:
                for u in course.get_users():
                    # Filter out teachers et al
                    if u.id not in env.CONFIG.excluded_users:
                        self.data[u.id] = u
        print( "Loaded {} students".format( len( self.data.keys() ) ) )

    # @property
    # def student_ids( self ):
    #     return list(set([k for k in self.data.keys()]))

    def get_student( self, canvas_id ):
        return self.data.get(canvas_id)
        # return Student(**s)
        # return Student(student_id=canvas_id, name=self.data[ canvas_id ])

    def get_student_name( self, canvas_id ):
        try:
            s = self.get_student( canvas_id )
            return s.name
            # return s['name']
        except KeyError:
            print('No student found for id: {}'.format(canvas_id))
            return ''
        except AttributeError:
            print('No student found for id: {}'.format(canvas_id))
            return ''

    def get_student_first_name( self, canvas_id ):
        full_name = self.get_student_name(canvas_id)

        if ',' in full_name:
            return full_name.split(',')[1]
        else:
            return full_name.split(' ')[0]

class StudentRepositoryOld( IRepo ):

    def __init__( self, course_ids=[] ):
        self.data = { }
        self.course_ids = course_ids
        # List of canvas api objects
        self.courses = []
        if len(course_ids) > 0:
            # option to use blank so can just load
            # test data from file manually
            for cid in course_ids:
                self.download( cid )

        print( "Loaded {} students".format( len( self.data.keys() ) ) )

    def _make_url( self, course_id ):
        """Makes the url for getting all students for course
        GET /api/v1/courses/:course_id/search_users """
        return make_url( course_id, 'search_users' )

    def download( self, course_id ):
        """Makes request to the server for all students enrolled in the
         course
        """
        data = { 'enrollment_type': 'student' }
        url = self._make_url( course_id )
        results = send_multi_page_get_request( url, data )
        self.store_results( results, course_id )

    def store_results( self, results_list, course_id ):
        """Stores student records in self.data with canvas id as keys"""
        for r in results_list:
            # Add the course id to the student result
            r[ 'course_id' ] = course_id
            # store it in data
            self.data[ r[ 'id' ] ] = r

    def get_student( self, canvas_id ):
        # s = self.data.get(canvas_id)
        # return Student(**s)
        return Student(student_id=canvas_id, name=self.data[ canvas_id ])

    def get_student_name( self, canvas_id ):
        try:
            s = self.get_student( canvas_id )
            # return s.name
            return s['name']
        except KeyError:
            return ''


if __name__ == '__main__':
    pass



"""
Created by adam on 3/16/20
"""
__author__ = 'adam'

import re

import pandas as pd

from CanvasHacks.Definitions.activity import Activity
from CanvasHacks.GradingCorrections.penalities import HalfLate, NoLatePenalty
from CanvasHacks.GradingMethods.nonempty import CreditForNonEmptyOLD
from CanvasHacks.Models.QuizModels import QuizDataMixin, StoredDataFileMixin

if __name__ == '__main__':
    pass


class TopicalAssignment( Activity, QuizDataMixin, StoredDataFileMixin ):
    title_base = 'Topical assignment'
    instructions_filename = 'topical-assignment-instructions.txt'
    creation_type = 'quiz'

    regex = re.compile( r"\bwarm up\b" )

    # regex = re.compile( r"\btopical assignment\b" )

    def __init__( self, **kwargs ):
        super().__init__( **kwargs )
        self.grace_period = pd.Timedelta( '2 days' )

        self.corrections = [ ]

        # The objects which will be used to penalize late assignments
        self.penalizers = [  HalfLate( self.due_at, self.grace_period ) ]
        # The object which will be used to penalize late assignments
        # todo deprecated
        self.penalizer = self.penalizers[ 0 ]

        # The object which will be used to assign the score
        # NB, min_words is 1 for now so as to not create problems with multiple-choice answers
        # This could be fixed in CAN-57
        # The object which will be used to assign the score
        self.grade_methods = [ CreditForNonEmptyOLD( min_words=2, count_stopwords=True ) ]
        self.grade_method = self.grade_methods[ 0 ]


class UnitEndSurvey( Activity ):
    """Representation of the survey at the end of each unit"""
    title_base = "Unit-end survey"
    instructions_filename = 'unit-end-survey-instructions.txt'
    regex = re.compile( r"\bunit-end survey\b" )
    creation_type = 'survey'

    def __init__( self, **kwargs ):
        super().__init__( **kwargs )


        self.corrections = [ ]

        # The objects which will be used to penalize late assignments
        self.penalizers = [ NoLatePenalty(  ) ]
        # The object which will be used to penalize late assignments
        # todo deprecated
        self.penalizer = self.penalizers[ 0 ]

        # The object which will be used to assign the score
        # NB, min_words is 1 for now so as to not create problems with multiple-choice answers
        # This could be fixed in CAN-57
        # The object which will be used to assign the score
        self.grade_methods = [ CreditForNonEmptyOLD( min_words=2, count_stopwords=True ) ]
        self.grade_method = self.grade_methods[ 0 ]



"""
Created by adam on 12/24/19
"""
__author__ = 'adam'

if __name__ == '__main__':
    pass
import random
from unittest.mock import MagicMock

import pandas as pd
import pytz
from faker import Faker

fake = Faker()
from CanvasHacks.Definitions.skaa import InitialWork, Review, MetaReview
from CanvasHacks.Definitions.discussion import DiscussionForum, DiscussionReview
from CanvasHacks.Definitions.unit import Unit

from CanvasHacks.PeerReviewed.Submissions import SubmissionFactory
from tests.factories.CanvasApiFactories import peer_review_result_factory, submission_result_factory


def activity_data_factory( name=None ):
    return {
        'id': random.randint( 1, 9999999999 ),
        'name': fake.word() if name is None else name,
        'access_code': fake.word(),
        'email_intro': fake.text(),
        'email_subject': fake.text(),
        'html_url': fake.url(),
        'open_at': pd.to_datetime( fake.date_time_this_century( before_now=True, after_now=False, tzinfo=pytz.utc ) ),
        'due_at': pd.to_datetime( fake.date_time_this_century( before_now=False, after_now=True, tzinfo=pytz.utc ) ),
        'lock_at': pd.to_datetime( fake.date_time_this_century( before_now=False, after_now=True, tzinfo=pytz.utc ) ),
        'completion_points': random.randint( 0, 1000 ),
        'max_points': random.randint( 0, 1000 ),
    }


def test_data_factory():
    return {
        'initial': activity_data_factory(),
        'review': activity_data_factory(),
        'metareview': activity_data_factory(),
        'discussion_forum': activity_data_factory(),
        'discussion_review': activity_data_factory()
    }


def unit_factory( course=None, unit_number=None, initial_is_quiz_type=True ):
    """Creates a Unit object with fake data for all assignments"""
    course = course if course is not None else MagicMock()
    unit_number = unit_number if unit_number is not None else random.randint( 1, 22222 )
    test_data = test_data_factory()
    initial = InitialWork( **test_data[ 'initial' ] )
    initial.id = random.randint( 0, 10000 )
    if initial_is_quiz_type:
        initial.quiz_id = initial.id
    # review
    review = Review( **test_data[ 'review' ] )
    review.id = random.randint( 0, 10000 )
    review.quiz_id = review.id
    review.access_code = fake.ean8()
    review.activity_link = fake.uri()
    # metareview
    meta = MetaReview( **test_data[ 'metareview' ] )
    meta.id = random.randint( 0, 10000 )
    meta.access_code = fake.ean8()
    meta.quiz_id = meta.id
    # discussion forum
    discussion_forum = DiscussionForum( **test_data[ 'discussion_forum' ] )
    discussion_forum.id = random.randint( 0, 10000 )
    discussion_forum.discussion_topic = {'id' : discussion_forum.id}
    discussion_forum.access_code = fake.ean8()
    # discussion review
    discussion_review = DiscussionReview( **test_data[ 'discussion_review' ] )
    discussion_review.id = random.randint( 0, 10000 )
    discussion_review.access_code = fake.ean8()

    unit = Unit( course, unit_number )
    unit.components = [ initial, review, meta, discussion_forum, discussion_review ]
    return unit


def submissions_factory( student1, student2, assignment ):
    submissions = [ ]
    # initial work
    sfac1 = SubmissionFactory( assignment.initial_work )
    r1 = submission_result_factory( assignment.initial_work, student1 )
    submissions.append( sfac1.make( r1 ) )

    r2 = submission_result_factory( assignment.initial_work, student2 )
    submissions.append( sfac1.make( r2 ) )

    # reviews
    sfac2 = SubmissionFactory( assignment.review )
    pr1 = peer_review_result_factory( student1, student2 )
    submissions.append( sfac2.make( pr1 ) )

    pr2 = peer_review_result_factory( student2, student1 )
    submissions.append( sfac2.make( pr2 ) )

    # meta reviews
    sfac3 = SubmissionFactory( assignment.metareview )
    mr1 = peer_review_result_factory( student1, student2 )
    submissions.append( sfac3.make( mr1 ) )

    mr2 = peer_review_result_factory( student2, student1 )
    submissions.append( sfac3.make( mr2 ) )

    return submissions


def discussion_entry_factory( **kwargs ):
    """ Returns something like:
    {'id': 2132485, 'user_id': 169155,
    'parent_id': None, 'created_at': '2020-01-16T23:01:53Z',
    'updated_at': '2020-01-16T23:01:53Z', 'rating_count': None,
    'rating_sum': None, 'user_name': 'Test Student',
    'message': '<p>got em</p>', 'user': {'id': 169155,
    'display_name': 'Test Student',
    'avatar_image_url': 'https://canvas.csun.edu/images/messages/avatar-50.png',
    'html_url': 'https://canvas.csun.edu/courses/85210/users/169155',
    'pronouns': None, 'fake_student': True},
    'read_state': 'unread', 'forced_read_state': False,
    'discussion_id': 737847, 'course_id': 85210}
    """
    dummy = { 'id': random.randint( 0, 100000 ),
              'user_id': random.randint( 0, 10000 ),
              'parent_id': None,
              'created_at': pd.to_datetime(
                  fake.date_time_this_century( before_now=False, after_now=True, tzinfo=None ) ),
              'updated_at': pd.to_datetime(
                  fake.date_time_this_century( before_now=False, after_now=True, tzinfo=None ) ),
              'rating_count': None,
              'rating_sum': None,
              'user_name': fake.name(),
              'message': '<p>{}</p>'.format( fake.paragraph() ),
              'read_state': 'unread', 'forced_read_state': False,
              'discussion_id': random.randint( 0, 10000 ),
              'course_id': random.randint( 0, 10000 )
              }

    # if stuff passed in, replace
    for k in kwargs.keys():
        dummy[ k ] = kwargs[ k ]
    return dummy


def content_repo_data_object_factory( **kwargs ):
    """Creates a dictionary with the keys representing columns
    that the dataframe in self.data will be expected to have """
    d = {
        'student_id': fake.random.randint( 11111, 999999999 ),
        'workflow_state': 'submitted'
    }

    for k, v in kwargs.items():
        d[ k ] = v
    return d



"""
Created by adam on 3/11/20
"""
__author__ = 'adam'

from unittest import TestCase

from CanvasHacks.Processors.cleaners import HtmlCleaner
from TestingBase import TestingBase

if __name__ == '__main__':
    pass

TEST_DATA = [
    { 'input': "<p>Tacos are nom</p>",
      'expect': "Tacos are nom"
      },

]


class TestHtmlCleaner( TestingBase ):

    def setUp(self) -> None:
        self.config_for_test()
        self.obj = HtmlCleaner()

    def test_clean( self ):
        for t in TEST_DATA:
            self.assertEqual(t['expect'], self.obj.clean(t['input']))



"""
Created by adam on 3/2/20
"""
__author__ = 'adam'

from unittest.mock import MagicMock, patch

from faker import Faker

from CanvasHacks.DAOs.sqlite_dao import SqliteDAO
from CanvasHacks.Repositories.status import InvitationStatusRepository
from CanvasHacks.SkaaSteps.SendForumPostsToReviewer import SendForumPostsToReviewer
from tests.TestingBase import TestingBase
from tests.factories.ModelFactories import student_factory
from tests.factories.PeerReviewedFactories import unit_factory
from tests.factories.RepositoryMocks import ContentRepositoryMock

fake = Faker()

if __name__ == '__main__':
    pass


class TestUnitTests( TestingBase ):

    def setUp( self ):
        self.config_for_test()
        self.unit = unit_factory()
        self.activity = self.unit.discussion_forum
        self.course = MagicMock()

        # student recieiving the message
        self.author = student_factory()
        self.reviewer = student_factory()

        # This would be the content unit
        self.work = fake.text()
        # Set to none so that loader thinks not a quiz
        self.activity_id = self.unit.discussion_forum.id
        self.dao = SqliteDAO()
        self.session = self.dao.session
        self.create_new_and_preexisting_students()
        # Prepare fake work repo to give values to calling  objects

        # self.studentRepo = StudentRepository()
        # self.studentRepo.get_student = MagicMock( return_value=self.reviewer )
        self.contentRepo = ContentRepositoryMock()
        self.contentRepo.get_formatted_work_by = MagicMock( return_value=self.work )
        self.review_assign = MagicMock( assessor_id=self.reviewer.id, assessee_id=self.author.id )
        self.statusRepo = MagicMock()
        # Prepare fake work repo to give values to calling  objects
        self.workRepo = ContentRepositoryMock()
        self.workRepo.create_test_content( self.student_ids )
        self.workRepo.add_students_to_data( self.student_ids, make_dataframe=True )

    def test_instantiates_correct_status_repo( self ):
        """The sending of metareview results requires a
        special status repository"""
        obj = SendForumPostsToReviewer( course=self.course, unit=self.unit, is_test=True, send=True )
        self.assertEqual(len(obj.statusRepos), 1, "expected number of status repos")
        self.assertIsInstance( obj.statusRepos[0], InvitationStatusRepository,
                               "Correct status repo instantiated" )

    def test__message_step( self ):
        self.skipTest( "todo" )

    def test__assign_step( self ):
        self.skipTest( "todo" )

    @patch( 'CanvasHacks.SkaaSteps.ISkaaSteps.DisplayManager' )
    @patch( 'CanvasHacks.SkaaSteps.SendForumPostsToReviewer.WorkRepositoryLoaderFactory' )
    def test__load_step( self, workLoaderMock, displayManagerMock ):
        self.workRepo.remove_student_records = MagicMock()
        workLoaderMock.make = MagicMock( return_value=self.workRepo )

        obj = SendForumPostsToReviewer( course=self.course, unit=self.unit, is_test=True, send=True )

        # call
        obj._load_step()

        # check
        workLoaderMock.make.assert_called()
        displayManagerMock.assert_called()

        call_args = [ c[ 0 ][ 0 ] for c in workLoaderMock.call_args_list ]
        # print( call_args )
        # self.assertEqual()


class TestFunctionalTests( TestingBase ):

    def setUp( self ):
        self.config_for_test()
        self.unit = unit_factory()
        self.activity = self.unit.discussion_forum
        self.course = MagicMock()

        # student recieiving the message
        self.author = student_factory()
        self.reviewer = student_factory()

        # This would be the content unit
        self.work = fake.text()
        # Set to none so that loader thinks not a quiz
        self.activity_id = self.unit.discussion_forum.id
        self.dao = SqliteDAO()
        self.session = self.dao.session
        self.create_new_and_preexisting_students()
        # Prepare fake work repo to give values to calling  objects

        # self.studentRepo = StudentRepository()
        # self.studentRepo.get_student = MagicMock( return_value=self.reviewer )
        self.contentRepo = ContentRepositoryMock()
        self.contentRepo.get_formatted_work_by = MagicMock( return_value=self.work )
        self.review_assign = MagicMock( assessor_id=self.reviewer.id, assessee_id=self.author.id )
        self.statusRepo = MagicMock()
        # Prepare fake work repo to give values to calling  objects
        self.workRepo = ContentRepositoryMock()
        self.workRepo.create_test_content( self.student_ids )
        self.workRepo.add_students_to_data( self.student_ids, make_dataframe=True )

    @patch( 'CanvasHacks.SkaaSteps.SendForumPostsToReviewer.InvitationStatusRepository' )
    @patch( 'CanvasHacks.Messaging.base.ConversationMessageSender.send' )
    @patch( 'CanvasHacks.SkaaSteps.ISkaaSteps.StudentRepository' )
    @patch( 'CanvasHacks.SkaaSteps.SendForumPostsToReviewer.WorkRepositoryLoaderFactory' )
    def test_run( self, workLoaderMock, studentRepoMock, messengerMock, statusRepoMock ):
        self.workRepo.submitter_ids = self.new_students_ids

        workLoaderMock.make = MagicMock( return_value=self.workRepo )

        # prepare student repo
        students = { s.student_id: s for s in self.students }

        def se( sid ):
            return students.get( sid )

        # studentRepoMock.get_student = MagicMock(side_effect=se)
        # studentRepoMock.download = MagicMock( return_value=self.students )

        # call
        obj = SendForumPostsToReviewer( course=self.course, unit=self.unit, is_test=True, send=True )
        # obj.studentRepo = MagicMock()
        obj.studentRepo.get_student = MagicMock( side_effect=se )
        obj.studentRepo.download = MagicMock( return_value=self.students )
        preexisting_pairings = self.create_preexisting_review_pairings( self.activity_id, self.preexisting_students,
                                                                        obj.dao.session )

        obj.run()

        # check
        self.assertEqual( len( obj.associations ), len( self.new_students_ids ),
                          "Correct number of students assigned" )

        # Check that new assignments don't involve previously assigned sudents
        for rec in obj.associations:
            self.assertNotIn( rec.assessor_id, self.preexisting_student_ids,
                              "Newly assigned assessor not among previously assigned students" )
            self.assertNotIn( rec.assessee_id, self.preexisting_student_ids,
                              "Newly assigned assessee not among previously assigned students" )

        # Check whether each new student has been assigned
        new_assessor_ids = [ r.assessor_id for r in obj.associations ]
        new_assessee_ids = [ r.assessee_id for r in obj.associations ]
        self.assertEqual( len( set( new_assessor_ids ) ), len( self.new_students_ids ),
                          "No duplicate assessor assignments" )
        self.assertEqual( len( set( new_assessee_ids ) ), len( self.new_students_ids ),
                          "No duplicate assessee assignments" )
        for sid in self.new_students_ids:
            self.assertIn( sid, new_assessee_ids, "Student is an assessee" )
            self.assertIn( sid, new_assessor_ids, "Student is an assessor" )

        # ================== Events on Messenger
        # Check that mocked objects were called with expected data
        messengerMock.assert_called()
        self.assertEqual( messengerMock.call_count, len( self.new_students_ids ),
                          "Send method called expected number of times" )
        messenger_args = [ (c[ 1 ][ 'student_id' ], c[ 1 ][ 'subject' ], c[ 1 ][ 'body' ]) for c in
                           messengerMock.call_args_list ]
        # print(args)

        # Check that all messages have the correct subject
        for sid, subj, body in messenger_args:
            self.assertEqual( self.unit.discussion_review.email_subject, subj, "Correct subject line" )

        # Status repo calls on messenger
        obj.messenger.status_repositories[0].record.assert_called()
        call_list = obj.messenger.status_repositories[0].record.call_args_list

        # obj.messenger.status_repositories.record_invited.assert_called()
        # call_list = obj.messenger.status_repositories.record_invited.call_args_list
        status_args = [ c[ 0 ][ 0 ] for c in call_list ]
        self.assertEqual( len( self.new_students ), len( call_list ),
                          "Status repo record_invited called expected number of times" )
        for sid in self.new_students_ids:
            self.assertIn( sid, status_args, "StatusRepo.record_invited called on all students" )

        # student repo calls on messenger
        for sid in self.new_students_ids:
            obj.messenger.student_repository.get_student.assert_any_call( sid )

        # Check the content sent
        for record in obj.associations:
            # print(record.assessee_id)
            author_text = self.workRepo.get_formatted_work_by( record.assessee_id )
            # see if sent to assessor
            sent_text = [ t[ 2 ] for t in messenger_args if t[ 0 ] == record.assessor_id ][ 0 ]
            rx = r'{}'.format( author_text )
            self.assertRegex( sent_text, rx, "Author's work in message sent to reviewer" )

    def test_audit_frame( self ):
        self.skipTest( "todo" )



"""
Created by adam on 9/20/18
"""
__author__ = 'adam'

import requests
from requests.exceptions import HTTPError

from CanvasHacks import environment
from CanvasHacks.Api.UrlTools import make_url


def make_request_header():
    """Creates the request header with Authorization value expected by canvas"""
    return { 'Authorization': 'Bearer {}'.format( environment.CONFIG.canvas_token ) }


def send_get_request( url, data={ } ):
    """Makes a get request to the given url, using the data.
    Returns a dictionary from the json
    """
    head = { 'Authorization': 'Bearer {}'.format( environment.CONFIG.canvas_token ) }
    response = requests.get( url, headers=make_request_header(), json=data )
    return response.json()


def send_multi_page_get_request(url, data={}, per_page=45):
    responses = []

    url = "{}/?per_page={}".format(url, per_page)

    try:
        while True:
            print( url )
            response = requests.get( url, headers=make_request_header(), json=data )
            # If the response was successful, no Exception will be raised
            response.raise_for_status()
            # Continuing on since was successful
            responses += response.json()
            url = response.links[ 'next' ][ 'url' ]

    except KeyError:
        return responses

    except HTTPError as http_err:
        print(f'HTTP error occurred: {http_err}')  # Python 3.6
        return responses

    except Exception as err:
        print(f'Other error occurred: {err}')  # Python 3.6
        return responses



def send_post_request( url, data ):
    """Makes a post request to the given url, using the data.
    Returns a dictionary from the json
    """
    head = { 'Authorization': 'Bearer {}'.format( environment.CONFIG.canvas_token ) }
    response = requests.post( url, headers=head, json=data )
    return response.json()


def send_put_request( url, data ):
    """Makes a put request to the given url, using the data.
    Returns a dictionary from the json
    """
    response = requests.put( url, headers=make_request_header(), json=data )
    return response.json()


# Assignments
# Get unit list
def get_all_course_assignments( course_id ):
    """Returns a list of all the assignments for the course
    Uses api: GET /api/v1/courses/:course_id/assignments
    """
    url = make_url( course_id, 'assignments' )
    url = "%s?per_page=50" % url
    return send_get_request( url, { 'include': 'submissions' } )


def get_assignment( course_id, assignment_id ):
    """Retrieves the specified unit and returns it as a dictionary"""
    url = make_url( course_id, 'assignments' )
    url = "%s/%s" % (url, assignment_id)
    #     print(url)
    return send_get_request( url )


def get_assignments_needing_grading( course_id ):
    """Returns a list of tuples (name, id) of assignments which
    have at least one ungraded submission"""
    assigns = get_all_course_assignments( course_id )
    assigns = [ a for a in assigns if a[ 'needs_grading_count' ] > 0 ]

    # to_grade = [ (g[ 'name' ].strip(), g[ 'id' ]) for g in to_grade ]
    return assigns



def get_assignments_with_submissions( course_id, needs_grading=True ):
    """Queries the server and returns only the assignments
    in the course which at least one
    student has submitted."""
    assignments = get_all_course_assignments( course_id )
    if needs_grading:
        assignments = [ a for a in assignments if a[ 'needs_grading_count' ] > 0 ]
    assignments = [ a for a in assignments if a[ "has_submitted_submissions" ] is True ]
    return assignments



def get_all_course_quizzes(course_id):
    """ GET /api/v1/courses/:course_id/quizzes """
    url = make_url(course_id, 'quizzes')
    #     print(url)
    return send_get_request(url)


def get_all_quiz_submissions(course_id, quiz_id, per_page=42):
    """
    GET /api/v1/courses/:course_id/quizzes/:quiz_id/submissions
    """
    url = make_url(course_id, 'quizzes')
    url += "/%s/submissions?per_page=%s" % (quiz_id, per_page)

    print(url)
    #     return send_get_request(url)
    responses = [ ]
    #     url = make_url( course_id, assign_type )
    #     url = "%s/%s/submissions?per_page=%s" % (url, quiz_id, per_page)
    try:
        while True:
            print( url )
            response = requests.get( url, headers=make_request_header() )
            responses += response.json()['quiz_submissions']
            url = response.links[ 'next' ][ 'url' ]
    except KeyError:
        return responses


if __name__ == '__main__':
    pass



"""
This contains tools for displaying all student submitted
text along with toggle buttons for assigning credit or
no credit

For testing

# credit_list = [0, 2]

# make_consolidated_text_fields(test_results, credit_list)

Created by adam on 2/5/19
"""
__author__ = 'adam'
import json

from IPython.display import display
from ipywidgets import widgets


def make_assignment_header( store ):
    """Displays the header for each unit"""
    entry = """
    <h1>{}</h1>
    <h2>Class id: {}</h2>"""
    return display( widgets.HTML( entry.format( store.assignment_name, store.course_id ) ) )


def make_text_display( student_id, text, student_name='' ):
    """Handles the formatting of the student's text"""
    entry = """
          <div id='%s'>
            <h3>%s</h3>
            <p>
                %s
            </p>
        </div>"""
    student_name = "{}  |  ".format(student_name) if len(student_name) > 1 else student_name
    header = "{}{}".format(student_name, student_id)
    e = entry % (student_id, header, text)
    return widgets.HTML( e )


def make_submission_output( text, student_id, credit_list, studentRepository=None ):
    """Creates the display of the submitted text with a toggle button to
    update whether the student receives credit
    """
    bval = 'Credit' if student_id in credit_list else 'No credit'
    credit_button = widgets.ToggleButtons(
        options=[ 'Credit', 'No credit' ],
        value=bval,
        description='',
        disabled=False
    )
    student_name = ''

    if studentRepository:
        student_name = studentRepository.get_student_name(student_id)

    ta = make_text_display( student_id, text, student_name )

    def on_value_change( change ):
        """The relevant callback
        NB the use of scope to define student_id"""
        v = change[ 'new' ]
        print( v, student_id )
        try:
            if v == 'Credit':
                credit_list.append( student_id )
            elif v == 'No credit':
                credit_list.remove( student_id )
        except ValueError:
            pass

    credit_button.observe( on_value_change, names='value' )
    display( widgets.VBox( [ ta, credit_button ] ) )


def make_consolidated_text_fields( store, studentRepository=None ):
    """Displays each entry with a toggle to adjust whether the
    student receives credit"""
    for r in store.submissions:
        make_submission_output( r[ 'body' ], r[ 'student_id' ], store.credit, studentRepository )


def make_consolidated_text_file( journal_folder, filename='compiled-text.txt', studentRepository=None ):
    """Reads the data from the relevant json file and then
    writes each entry in to a single text file for ease of reading
    todo: This should be refactored out of the widgets file
    """
    entry = """
    {student_id} \n
    {body} \n
    \n
    """
    to_write = "%s/%s" % (journal_folder, filename)
    with open( "%s/all-submissions.json" % journal_folder, 'r' ) as f:
        j = json.load( f )
        with open( to_write, 'w+' ) as cf:
            for rec in j:
                txt = entry.format( **rec )
                if studentRepository:
                    name = studentRepository.get_student_name(rec['student_id'])
                    txt = "{} \n {}".format(name, txt)
                #     print(entry.format(**rec))
                cf.write(  txt )

    print( "Consolidated file written to {}".format( to_write ) )


def make_test_data( number_students ):
    return [ { 'student_id': i, 'body': 'Body text for student {}'.format( i ) } for i in range( 0, number_students ) ]


if __name__ == '__main__':
    pass



"""
Created by adam on 3/15/20
"""
__author__ = 'adam'

import nltk

from CanvasHacks.Models.model import StoreMixin
from CanvasHacks.Text.process import WordbagMaker

if __name__ == '__main__':
    pass


class ITextStat:

    def analyze( self, content ):
        raise NotImplementedError


class WordCount(ITextStat):

    def __init__(self, **kwargs):
        """
        kwargs will often contain
            count_stopwords
        :param kwargs:
        """
        self.bagmaker = WordbagMaker(**kwargs)

    def analyze( self, content ):
        """
        Returns the total number of words in the content
        :param content:
        :return:
        """
        bag = self.bagmaker.process(content)
        return len( bag )


class WordFreq(object):
    """
    Tools for computing and plotting frequencies of word appearances using the nltk.FreqDist method
    """

    def __init__(self, word_list):
        """
        Args:
            word_list: list of words
        """
        self.data = word_list
        self.unique = sorted(set(self.data))

        assert(type(word_list) is list)
        self.freqDist = nltk.FreqDist(self.data)
        self.ranking = list(self.freqDist.keys())

    def topN(self, number_to_display):
        """
        Returns the N most common items in the dataset

        Args:
            number_to_display: The number to display
        """
        assert(type(number_to_display) is int)
        return self.ranking[0: number_to_display]

    def plot(self, number_to_display, **kwargs):
        """
        Displays a plot of the frequency distribution of item frequencies
        Args:
            number_to_display: The top n to display
        """
        assert(type(number_to_display) is int)
        self.freqDist.plot(number_to_display, cumulative=True, **kwargs)

    @property
    def word_freq_dicts( self ):
        """
        A list of dictionaries with the frequency of each word
        Returns:
            Dictionary with words as keys and 'word' and 'count'
        """
        # fd = nltk.FreqDist(self.data)
        # for word in self.data:
        #     fd.inc(word)
        # results = []
        return [{'word': w[0], 'count' : w[1]} for w in self.freqDist.items()]
        #     results.append()
        # return results


"""
Created by adam on 2/26/20
"""
__author__ = 'adam'

from CanvasHacks.Errors.data_ingestion import NoWorkDownloaded
from CanvasHacks.Loaders.assignment import AssignmentDownloadLoader
from CanvasHacks.Loaders.discussion import DiscussionFileLoader, DiscussionDownloadLoader
from CanvasHacks.Loaders.interfaces import ILoaderFactory
from CanvasHacks.Loaders.quiz import AllQuizReportDownloader, AllQuizReportFileLoader, NewQuizReportDownloadLoader,\
    NewQuizReportFileLoader, QuizComboLoader

if __name__ == '__main__':
    pass


class LoaderFactory( ILoaderFactory ):
    """Decides which loader to use
    Makes the initial determination on whether is_non_quiz is set
    as an arg
    """

    @staticmethod
    def make( *args, **kwargs ):
        """
        Should use kwarg is_non_quiz=True if want
        an AssignmentLoader (for non-quizzes)
        :param args:
        :param kwargs:
        :return:
        """
        try:
            if kwargs['is_discussion']:
                return DiscussionLoaderFactory.make(*args, **kwargs)
        except KeyError:
            # Fall through to next set
            pass

        try:
            if kwargs[ 'is_quiz' ]:
                return QuizLoaderFactory.make( *args, **kwargs )
            # If key is set and false
            return AssignmentLoaderFactory.make( *args, **kwargs )

        except KeyError:
            # This is the default since most things use quizzes
            return QuizLoaderFactory.make( *args, **kwargs )


class AssignmentLoaderFactory( ILoaderFactory ):
    """Decides which loader to use for a non-quiz unit"""

    @staticmethod
    def make( **kwargs ):
        return AssignmentDownloadLoader


class QuizLoaderFactory( ILoaderFactory ):
    """
    Decides which form of quiz loader to use
    """

    @staticmethod
    def make( download=True, only_new=False, combo=False, **kwargs ):
        if combo:
            return QuizComboLoader

        if download and only_new:
            return NewQuizReportDownloadLoader

        if download:
            return AllQuizReportDownloader

        # We're just to load from file
        if only_new and only_new:
            return NewQuizReportFileLoader

        return AllQuizReportFileLoader


class DiscussionLoaderFactory(ILoaderFactory):

    @staticmethod
    def make( download=True, **kwargs ):

        if download:
            return DiscussionDownloadLoader

        return DiscussionFileLoader



"""
Created by adam on 3/15/20
"""
__author__ = 'adam'


from CanvasHacks.GradingMethods.base import IGradingMethod
from CanvasHacks.Processors.cleaners import TextCleaner
from CanvasHacks.Text.stats import WordCount

if __name__ == '__main__':
    pass


def make_pct_required_count_thresholds( required_count, step=10 ):
    return [
        { 'min_count': round( required_count * pct * 0.01 ), 'pct_credit': round( pct * 0.01 ) } for pct in
        range( 0, 100, step ) ]


class GradeByWordCount( IGradingMethod):

    def __init__( self, threshold_dicts=[ ], required_count=None, pct_of_score=1, count_stopwords=True ):
        """
        Count dict defines the thresholds via wordcounts
        and percentage of total. Should be a list like:
        [
            { count : int,
            pct_credit : float
            },
            {count : 1000,
            pct_credit: 54.3
            }
        ]
        :param threshold_dicts:
        :param count_stopwords: Whether stopwords should be included in the wordcount
        """

        self.required_count = required_count
        self.threshold_dicts = threshold_dicts
        self.count_stopwords = count_stopwords

        # Will remove html and most encoding artifacts.
        # Whatever we're calling this on, should've already
        # removed that stuff, but just to be safe
        self.cleaner = TextCleaner()

        # The object which will handle the actual processing and computation
        self.analyzer = WordCount( count_stopwords=count_stopwords )

        if self.required_count is not None and len( self.threshold_dicts ) > 0:
            self.make_pct_required_count_thresholds( self.required_count )

        self.prepare_dicts()

    def make_pct_required_count_thresholds( self, required_count, step=10 ):
        self.threshold_dicts = make_pct_required_count_thresholds(required_count, step)
        # [
        #     { 'min_count': round( required_count * pct * 0.01 ), 'pct_credit': round( pct * 0.01 ) } for pct in
        #     range( 0, 100, step ) ]
        self.prepare_dicts()
        return self.threshold_dicts

    def prepare_dicts( self ):
        # Validate and put the dicts we've received into the
        # correct order
        # if len( self.threshold_dicts ) == 0:
        #     raise InvalidGradingValuesError
        self.threshold_dicts.sort( key=lambda x: x[ 'min_count' ], reverse=True )

    def grade( self, content, **kwargs ):
        """
        Returns the pct credit as a float between 0-100
        Assumes that html and encoding artifacts have been
        separately removed

        :param content:
        :param kwargs:
        :return:
        """
        # Remove html and other artifacts
        # content = self.cleaner.clean( content )

        # Tokenize and count
        word_count = self.analyzer.analyze( content )

        # Lookup correct pct total
        for td in self.threshold_dicts:
            if word_count >= td[ 'min_count' ]:
                return td[ 'pct_credit' ]

        # if we've fallen all the way through the dicts, they
        # are below the lowest cutoff
        return 0



"""
Tools for uploading scores to canvas for journals

Created by adam on 1/29/19
"""
__author__ = 'adam'

import requests
from IPython.display import display
from ipywidgets import widgets

from CanvasHacks.Repositories.DataManagement import DataStore, DataStoreNew
from CanvasHacks.Api.RequestTools import make_request_header
from CanvasHacks.Api.UrlTools import make_url


def upload_students_receiving_credit( store: DataStore ):
    """Makes the requests to the server to assign each of the students credit
    for the unit
    Hits https://canvas.instructure.com/doc/api/submissions.html#method.submissions_api.update
    """
    if isinstance(store, DataStoreNew):
        print("uploading from new data store")
        for sub, score in store.results:
            sid = sub.user_id
            upload_credit(store.course_id, store.assignment_id, sid, score)
    else:
        for sid in store.credit:
            upload_credit(store.course_id, store.assignment_id, sid, 100)


def upload_credit( course_id, assignment_id, student_id, pct_credit ):
    """
    Handles actual upload to server.

    Hits https://canvas.instructure.com/doc/api/submissions.html#method.submissions_api.update
    :param pct_credit: 100 for full
    :param course_id:
    :param assignment_id:
    :param student_id:
    :return:
    """
    pct = "{}%".format(pct_credit) if isinstance(pct_credit, int) or pct_credit[-1:] != '%' else pct_credit

    data = { 'submission': { 'posted_grade': pct } }

    url_temp = "{url_base}/{assignment_id}/submissions/{student_id}"

    url = url_temp.format( url_base=make_url( course_id, 'assignments' ),
                           assignment_id=assignment_id,
                           student_id=student_id )

    print( 'Uploading {} credit'.format(pct), url )
    requests.put( url, headers=make_request_header(), json=data )


def assign_no_credit( course_id, assignment_id, students ):
    """Makes the requests to the server to assign each of the students no credit
    for the unit
    """
    data = { 'submission': { 'posted_grade': 'fail' } }

    url = make_url( course_id, 'assignments' )
    url = "%s/%s/submissions" % (url, assignment_id)

    for s in students:
        surl = "%s/%s" % (url, s)
        print( 'no credit', surl )


#         requests.put(surl, headers=make_request_header(), json=data)


def make_upload_button( store ):
    """Displays a button for each unit. When the button
    is clicked it uploads grades for all students in the complete list
    The button's styling changes while uploading and again when complete
    """

    def upload_callback( event ):
        # change button style to indicate working
        b.button_style = 'warning'
        # upload grades for students receiving credit
        upload_students_receiving_credit( store )
        # change button style
        b.button_style = 'success'

    desc = 'Upload grades for {}'.format( store.assignment_name )
    layout = widgets.Layout( width='50%' )
    b = widgets.Button( description=desc, button_style='danger', layout=layout )
    b.on_click( upload_callback )

    display( b )


if __name__ == '__main__':
    pass



"""
Created by adam on 2/25/20
"""
from functools import wraps

from CanvasHacks.Logging.Logging import StudentWorkLogger
from CanvasHacks.Logging.messages import MessageLogger

__author__ = 'adam'

if __name__ == '__main__':
    pass


def log_student_work( func ):
    """Decorator for writing to the STUDENT_WORK_PROCESSING_LOGNAME log"""

    @wraps( func )
    def wrapper( *args, **kwargs ):
        # handle logging
        StudentWorkLogger.write( "\n".join( args ) )
        # call og function
        func( *args, **kwargs )

    return wrapper


def log_message( func ):
    """Wraps function which sends messages to students
    and writes outgoing message info to log
    """

    @wraps( func )
    def wrapper( *args, **kwargs ):
        margs = "\n".join( [ str( a ) for a in args ] )
        mkwargs = "\n".join( [ "{} \t: {}".format( k, v ) for k, v in kwargs.items() ] )
        to_log = "{}\n{}".format( margs, mkwargs )
        # handle logging
        # MessageLogger.write( to_log )
        try:
            # call og function
            result = func( *args, **kwargs )
            if result:
                to_log += " \n ------- RESULT ------- \n "
                to_log += str( result )
                MessageLogger.write( to_log )
                return result
            MessageLogger.write( to_log )

        except Exception as e:
            to_log += " \n ------- ERROR ------- \n "
            to_log += e.__str__()
            MessageLogger.write( to_log, is_error=False )
            raise e

    return wrapper

    #

    # def __init__( self, log_file_path=.format() ):
    #     self.log = ''
    #     super().__init__( )
    #     self.log_file = log_file
    #     # self.UPATH = os.getenv("HOME")
    #     # self.log_file = '%s/Desktop/%s' % self.UPATH, log_file
    #     # self.log_file = "application_search.log"
    #     self.set_log_file( self.log_file )
    #
    # def write_to_file( self ):
    #     self.write( self.log )
    #     self.log = ''
    #
    # def run_start( self, run_number ):
    #     self.log += '\n --------------------------------- %s --------------------------- ' % datetime.now()
    #     self.log += "\n Run number: %d " % run_number
    #     self.write_to_file()
    #     print( self.log )
    #
    # def record_saver_action( self, savername, num_tweets ):
    #     """
    #     This will be called inside the observer to log a saver class action
    #     Args:
    #         savername: String name of the saver (mysql, redis, couchdb)
    #         num_tweets: Integer number of tweets saved
    #     """
    #     self.log += '\n         Called saver for %s to save %s tweets' % (savername, num_tweets)
    #     self.write_to_file()
    #


"""
Created by adam on 3/11/20
"""
__author__ = 'adam'

from CanvasHacks.Messaging.SendTools import ConversationMessageSender
from canvasapi.user import User

if __name__ == '__main__':
    pass

REVIEW_NON_SUBMITTER_TEMPLATE = """
Hi {name},

I hope everything is okay with you. It looks like you haven't yet submitted the {assignment_name} which was due {original_due_date}.

Please do this as soon as possible. The person you were assigned to review is waiting for your feedback; they cannot do the metareview until you have completed this assignment.

Please let me know if you have any questions. And, please, please, please get this turned in.

All best wishes,
/a

PS, If we've already spoken about this or if you've only recently been assigned the reviewing task, apologies for this automated reminder. 
"""


ESSAY_NON_SUBMITTER_TEMPLATE = """
Hi {name},

I hope everything is okay with you. It looks like you haven't yet submitted the {assignment_name} which was due {original_due_date}.

Missing one of these assignments means also missing points from the peer review and metareview. Thus I have re-opened the assignment until 11.59PM on {final_date}. Please take advantage of this and turn it in!

Once you turn it in, please let me know. If someone else has turned it in late (and you are definitely not the only one), I will pair you up so you can complete the review and metareview before 11.59PM on {final_date}

Please let me know if you have any questions. And, please, please, please get this turned in. I want you to succeed in this class!

All best wishes,
/a

PS, If we've already spoken about this, apologies for this automated reminder. 

"""

class EssayNonSubmittersMessaging:

    def __init__(self, unit, send=True):
        self.send = send
        self.unit = unit
        self.activity = self.unit.initial_work
        self.messenger = ConversationMessageSender()
        self.subject = "Missing Unit {} Essay".format(self.unit.unit_number)
        self.sent = []

    def prepare_message( self, student_name ):
        data = {
            'name': student_name,
            'assignment_name': self.activity.name,
            'original_due_date': self.activity.string_due_date,
            'final_date' : self.activity.string_lock_date
        }

        return ESSAY_NON_SUBMITTER_TEMPLATE.format(**data)

    def send_message_to_student( self, student_id, first_name):

        # student_id = student.id if isinstance(student, User) else student.student_id
        body = self.prepare_message(first_name)
        if self.send:
            msg = self.messenger.send(student_id=student_id, subject=self.subject, body=body)
            self.sent.append( (student_id, msg) )
            print( msg )
        else:
            self.sent.append((student_id, body))
            print(body)


class ReviewNonSubmittersMessaging:

    def __init__(self, unit, send=True):
        self.send = send
        self.unit = unit
        self.activity = self.unit.review
        self.messenger = ConversationMessageSender()
        self.subject = "Missing Unit {} Peer Review".format(self.unit.unit_number)
        self.sent = []

    def prepare_message( self, student_name ):
        data = {
            'name': student_name,
            'assignment_name': self.activity.name,
            'original_due_date': self.activity.string_due_date,
        }

        return REVIEW_NON_SUBMITTER_TEMPLATE.format(**data)

    def send_message_to_student( self, student_id, first_name):

        # student_id = student.id if isinstance(student, User) else student.student_id
        body = self.prepare_message(first_name)
        if self.send:
            msg = self.messenger.send(student_id=student_id, subject=self.subject, body=body)
            self.sent.append( (student_id, msg) )
            print( msg )
        else:
            self.sent.append((student_id, body))
            print(body)



"""
Created by adam on 3/16/20
"""
__author__ = 'adam'




if __name__ == '__main__':
    pass


class SkaaReviewGroup:
    """Activities which are part of a skaa"""
    pass


class DiscussionGroup:
    """Activities which are part of a discussion-review sequence"""
    pass

class ReviewType:
    """Activities which are reviews. Differentiated
    because may have multiple choice items and other differences
    """
    pass


"""
Created by adam on 3/24/20
"""

from CanvasHacks.Definitions.base import BlockableActivity
from CanvasHacks.Errors.grading import NonStringInContentField
from CanvasHacks.GradingAnalyzers.blocked import BlockedByOtherStudent
from CanvasHacks.GradingHandlers.base import IGrader
from CanvasHacks.Logging.penalties import PenaltyLogger
from CanvasHacks.Repositories.assignments import AssignmentRepository
from CanvasHacks.Repositories.interfaces import ISubmissionRepo

__author__ = 'adam'

if __name__ == '__main__':
    pass


class AssignmentGrader( IGrader ):
    # grade_method: IGradingMethod
    # penalizer: IPenalizer

    def __init__( self, work_repo: AssignmentRepository, submission_repo: ISubmissionRepo, association_repo=None,
                  **kwargs ):
        """

        :param work_repo:
        :param submission_repo:
        :param association_repo: Only needed if grading an assignment which can be blocked
        :param kwargs:
        """
        self.association_repo = association_repo
        self.blocked_checker = None
        self.work_repo = work_repo
        self.submission_repo = submission_repo
        super().__init__( **kwargs )

        self.corrections = self.activity.corrections

        self.penalizers = self.activity.penalizers

        self.grade_methods = self.activity.grade_methods
        # Todo Check that output of grading methods sums to 1?

        self.graded = [ ]

        if isinstance( self.work_repo.activity, BlockableActivity ) and self.association_repo is not None:
            # association repo better have been set
            # todo
            self.blocked_checker = BlockedByOtherStudent( self.submission_repo, self.association_repo )

    def _compute_score( self, content, **kwargs ):
        """
        Calls the grading method which calculates the points received for
        a given question.s
        :param content:
        :return: integer points, None, or on_empty
        """
        pct_possible = 0
        for method in self.grade_methods:
            # each method should return a float pct
            # all of which sum to 1
            pct_possible += method.grade( content )
        # pct_possible = self.grade_method.grade( content )

        # return pct_possible
        return self.work_repo.points_per_question * pct_possible

    def _compute_penalty_pct( self, row, **kwargs ):
        down = 0
        # get a negative float representing the pct the total score
        # should be adjusted down
        for penalizer in self.penalizers:
            down += penalizer.analyze( **row.to_dict(), **kwargs )
        return down

    def _compute_correction_pct( self, row, **kwargs ):
        up = 0
        for correction in self.corrections:
            # get a positive float representing the pct the total score
            # should be adjusted up
            up += correction.analyze( **row.to_dict(), **kwargs )
        return up

    def _compute_initial_total( self, row, **kwargs ):
        """
        Returns the questions dict to be sent and the total
        score
        :param row:
        :param kwargs:
        :return:
        """
        content = row[ 'body' ]
        return self._compute_question_score( content, **kwargs )

    # def _grade_row( self, row, **kwargs ):
    #     """Grades a row
    #
    #     NB, requires that the non-graded attempts be
    #     filtered out before passing in
    #
    #     todo test whether the problem with non-graded people getting zeros is caused here
    #
    #     :param row: pd.DataFrame row
    #     :param kwargs:
    #     :return: Dictionary formatted for uploading
    #     """
    #
    #     # for student
    #     total_score = 0
    #
    #     # compute the initial total score of the activity
    #     # this is the job of the GradingMethod
    #     total_score = self._compute_initial_total( row, **kwargs )
    #
    #     try:
    #         if total_score == 0 and self.blocked_checker is not None:
    #             # This will raise an exception if they are blocked
    #             # and we won't return anything
    #             self.blocked_checker.analyze( row[ 'student_id' ] )
    #
    #         # Compute penalty if needed
    #         # Will be 0 if not docking for lateness
    #         # Records of penalty will be stored on self.penalizer.penalized_records
    #         down = self._compute_penalty_pct( row, **kwargs )
    #
    #         # Calculate things which could increase the score
    #         # this also may raise an exception if the student was
    #         # unable to complete it
    #         up = self._compute_correction_pct( row, **kwargs )
    #
    #         adj = up + down
    #
    #         fudge_points = total_score * adj
    #
    #         out = self._make_graded_row_output( row, questions, fudge_points )
    #
    #         return out
    #
    #     except StudentUnableToComplete as e:
    #         # handle situation where they were blocked by e.g. a reviewer not turning
    #         # in their part
    #
    #         # NB this is going to be tricky once relax deadlines
    #         # Should initially just log everything. But there will need to
    #         # be a switch for when they get full credit
    #         print( e )
    #         if CanvasHacks.testglobals.TEST:
    #             # Reraise so can see what happened for tests
    #             raise e

    # def _make_graded_row_output( self, row, question_score_dict, fudge_points ):
    #     out = {
    #         'student_id': int( row[ 'student_id' ] ),
    #         'attempt': int( row[ 'attempt' ] ),
    #         'submission_id': int( row[ 'submission_id' ] ),
    #         'course_id': int( row[ 'course_id' ] ),
    #         'assignment_id': int( row[ 'assignment_id' ] ),
    #         'data': { }
    #     }
    #
    #     out[ 'data' ][ "assignment_submissions" ] = [
    #         {
    #             "attempt": int( row[ 'attempt' ] ),
    #             "fudge_points": fudge_points,
    #             "questions": question_score_dict
    #         }
    #     ]
    #
    #     return out
    #
    #     # if self.using_percentage:
    #     #     pct = 100 * adj
    #     #
    #     # # using total points
    #     # score = total_score * adj
    #
    #     #
    #     # # Grade on emptiness
    #     # for qid, column_name in self.work_repo.question_columns:
    #     #     content = row[ column_name ]
    #     #     pts = self._compute_question_score( content, **kwargs )
    #     #     # assignment type things require the scores to be uploaded
    #     #     # for each question.
    #     #     # Thus we grade each question and then penalize the
    #     #     # overall score with fudge points if necessary
    #     #     questions[ qid ] = { 'score': pts }
    #     #     total_score += pts
    #     #
    #     #     # Compute penalty if needed
    #     #     # Will be 0 if not docking for lateness
    #     #     # Records of penalty will be stored on self.penalizer.penalized_records
    #     #     fudge_points = self.penalizer.get_fudge_points(row['submitted'], total_score, row)
    #     #
    #     # out = self._make_graded_row_output(row, questions, fudge_points)
    #     #
    #     # return out

    def grade( self, **kwargs ):
        """
        Grades all rows in work_repo.data
        todo: Add logging of details of how grade assigned
        :return: List of objects ready for upload
        """
        # todo Add some sort of type check here
        for i, submission in self.work_repo.data.iterrows():
            # for submission in self.work_repo.data:
            if submission.body is not None:
                score = self._compute_score( submission.body )
                if score:
                    # todo reenable penalizer w correct checks
                    # down = self._compute_penalty_pct()
                    # score = self.penalizer.get_penalized_score( submission.submitted_at, score, record=submission )
                    score = int( score )
                    score = score if score <= 100 else 100
                    self.graded.append( (submission, score ) )

        # self.report_late_penalties()

        return self.graded

    def report_late_penalties( self ):
        # Report late penalties
        if len( self.penalizer.penalized_records ) > 0:
            for penalty_dict in self.penalizer.penalized_records:
                self._penalty_message( penalty_dict[ 'penalty' ], penalty_dict[ 'record' ] )

    def _penalty_message( self, penalty, record ):
        """
        Handles printing or logging of penalties applied

        # todo enable logging

        :param penalty:
        :param row: A Submission object
        :return:
        """
        stem = 'Student #{}: Submitted on {}; was due {}. Penalized {}'
        try:
            return stem.format( record.student_id, record.submitted, self.activity.due_at, penalty )
        except (TypeError, AttributeError):
            # Will hit this if a Submission object
            return stem.format( record.user_id, record.submitted_at, self.activity.due_at, penalty )

        for i, row in self.work_repo.data.iterrows():
            try:
                self.graded.append( self._grade_row( row, **kwargs ) )
            except NonStringInContentField as e:
                print( e, row )

        # Print or log any penalties applied.
        # The penalizer object leaves this task up to us
        PenaltyLogger.log( self.penalizer )

        return self.graded

    # def _penalty_message( self, penalty, row ):
    #     """
    #     Handles printing or logging of penalties applied
    #
    #     # todo enable logging
    #
    #     :param penalty:
    #     :param row:
    #     :return:
    #     """
    #     stem = 'Student #{}: Submitted on {}; was due {}. Penalized {}'
    #     return stem.format( row[ 'student_id' ], row[ 'submitted' ], self.activity.due_at, penalty )
    #

    # def report_late_penalties( self ):
    #     # Report late penalties
    #     if len( self.penalizer.penalized_records ) > 0:
    #         for penalty_dict in self.penalizer.penalized_records:
    #             self._penalty_message( penalty_dict[ 'penalty' ], penalty_dict[ 'record' ] )



"""
Created by adam on 1/29/19
"""
import json
import io
import PyPDF2
import docx
import requests
from requests.exceptions import HTTPError

from CanvasHacks.Api.RequestTools import make_request_header
from CanvasHacks.Api.UrlTools import make_url

__author__ = 'adam'

PICTURE_PLACEHOLDER = 'picture-uploaded picture-uploaded picture-uploaded picture-uploaded picture-uploaded '


def download_journals( url, destination ):
    """Handles making the request to download journals"""
    # download file
    print( url )
    file_response = requests.get( url, headers=make_request_header() )

    # write to folder
    print( "Writing to %s" % destination )
    open( destination, 'wb' ).write( file_response.content )


def download_submitted_file( url, filepath ):
    response = requests.get( url, headers=make_request_header() )
    # save it to file
    open( filepath, 'wb' ).write( response.content )


def get_submissions( course_id, assignment_id, assign_type='assignments', per_page=42 ):
    """Makes request to the server for all submissions for the given unit
    Example
        course_id = SECTION_930
        assignment_id = 288480
        response2 = get_submissions(course_id, assignment_id)
    """
    responses = [ ]
    url = make_url( course_id, assign_type )
    url = "%s/%s/submissions?per_page=%s" % (url, assignment_id, per_page)
    try:
        while True:
            print( url )
            response = requests.get( url, headers=make_request_header() )
            # If the response was successful, no Exception will be raised
            response.raise_for_status()
            # Continuing on since was successful
            responses += response.json()
            url = response.links[ 'next' ][ 'url' ]

    except KeyError:
        return responses

    except HTTPError as http_err:
        print(f'HTTP error occurred: {http_err}')  # Python 3.6
        return responses

    except Exception as err:
        print(f'Other error occurred: {err}')  # Python 3.6
        return responses





def process_response( response_json, journal_folder=None ):
    """Takes the response and pulls out submissions which used the text box, then downloads
    submitted files and processes out their text content
    """
    submissions = [ ]

    for j in response_json:
        result = { 'submission_id': j[ 'id' ], 'student_id': j[ 'user_id' ] }
        if j[ 'body' ] is not None:
            # The student used the online text box to submit
            result[ 'body' ] = j[ 'body' ]

        else:
            # The student submitted the journal as a separate document
            if 'attachments' in j.keys() and len( j[ 'attachments' ] ) > 0:
                url = j[ 'attachments' ][ 0 ][ 'url' ]
                filename = make_journal_filename( j )

                # download and save submitted file
                fpath = "%s/%s" % (journal_folder, filename)
                download_submitted_file( url, fpath )

                # open the file and extract text
                result[ 'body' ] = getBody( fpath )

            else:
                # NB., if a student never submitted (workflow_state = 'unsubmitted'),
                # then they will have an entry which lacks a body key
                result[ 'body' ] = None
        submissions.append( result )

    return submissions


def save_submission_json( submissions, folder, json_name='all-submissions' ):
    # save submissions
    with open( "%s/%s.json" % (folder, json_name), 'w' ) as fpp:
        json.dump( submissions, fpp )


def get_journal_filename( response ):
    """Extracts the name of the file submitted from the response"""
    return response[ 'attachments' ][ 0 ][ 'filename' ]


def make_journal_filename( response ):
    """Creates the standardized filename for saving"""
    return "%s-%s" % (response[ 'user_id' ], get_journal_filename( response ))


# -------------- Document processing handlers

def docx_handler( filename ):
    if type(filename) is bytes:
        doc = docx.Document(io.BytesIO(filename))
    else:
        doc = docx.Document( filename )
    fullText = [ ]
    for para in doc.paragraphs:
        fullText.append( para.text )

    return '\n'.join( fullText )


def pdf_handler( filepath ):
    """Extracts the text from a pdf file"""
    if type(filepath) is bytes:
        # if using online version and not saving
        # downloads to file
        pdfReader = PyPDF2.PdfFileReader( io.BytesIO(filepath))
    else:
        # If loading downloaded work from file
        pdfFileObj = open( filepath, 'rb' )
        pdfReader = PyPDF2.PdfFileReader( pdfFileObj )
    pageObj = pdfReader.getPage( 0 )
    return pageObj.extractText()


def picture_handler( v=None ):
    return PICTURE_PLACEHOLDER


def unknown_handler( v=None ):
    return 'u'
    # print( 'unknown', v)
    return None


handlers = {
    '.docx': docx_handler,
    '.doc': unknown_handler,
    '.pdf': pdf_handler,
    '.jpg': picture_handler,
    '.png': picture_handler,
    '.xml': picture_handler,
    '.pages': picture_handler,
    '.odt': picture_handler
}


def chooseHandler( url ):
    """Checks the end of the file download url to determine
    whether it matches any of the file types we have handlers
    registered for. Returns the relevant handler"""
    url = url.strip()
    lengths = [len(a) for a in handlers.keys()]
    for i in range( min(lengths), max(lengths) + 1):
        if url[ -i: ] in handlers.keys():
            return handlers.get( url[ -i: ])

    return unknown_handler


def getBody( filepath ):
    """Extracts the text from a saved file"""
    PICTURE_PLACEHOLDER = 'picture-uploaded picture-uploaded picture-uploaded picture-uploaded picture-uploaded '
    filepath = filepath.strip()

    if filepath[ -5: ] == '.docx':
        return docx_handler( filepath )

    if filepath[ -4: ] == '.pdf':
        return pdf_handler( filepath )

    elif filepath[ -4: ] == '.jpg':
        return PICTURE_PLACEHOLDER + filepath[ -4: ]

    elif filepath[ -4: ] == '.png':
        return PICTURE_PLACEHOLDER + filepath[ -4: ]

    elif filepath[ -4: ] == '.xml':
        return PICTURE_PLACEHOLDER + filepath[ -4: ]

    elif filepath[ -6: ] == '.pages':
        return PICTURE_PLACEHOLDER + filepath[ -6: ]

    elif filepath[ -4: ] == '.odt':
        return PICTURE_PLACEHOLDER + filepath[ -4: ]

    else:
        return PICTURE_PLACEHOLDER + filepath


def process_response_without_saving_files( response_json ):
    """Takes the response and pulls out submissions which used the text box, then downloads
    submitted files and processes out their text content
    """
    submissions = [ ]

    for j in response_json:
        result = { 'submission_id': j[ 'id' ], 'student_id': j[ 'user_id' ] }
        if j[ 'body' ] is not None:
            # The student used the online text box to submit
            result[ 'body' ] = j[ 'body' ]

        else:
            # The student submitted the journal as a separate document
            if 'attachments' in j.keys() and len( j[ 'attachments' ] ) > 0:
                url = j[ 'attachments' ][ 0 ][ 'url' ]
                # download the submitted file
                # print( "Downloading: ", url )
                response = requests.get( url, headers=make_request_header() )
                content = response.content

                # determine the appropriate handler to use
                filename = response.headers['content-disposition'].split('filename=')[1]
                filename = filename.replace('"', '')
                handler = chooseHandler(filename)

                # open the file and extract text
                result[ 'body' ] = handler( content )

            else:
                # NB., if a student never submitted (workflow_state = 'unsubmitted'),
                # then they will have an entry which lacks a body key
                result[ 'body' ] = None
        submissions.append( result )

    return submissions


def extract_body(submission):
    """Given a canvas api object returns the body text for the submission"""
    if submission.body is not None:
        # The student used the online text box to submit
        return submission.body

    else:
        # The student submitted the journal as a separate document
        if 'attachments' in submission.attributes.keys() and len( submission.attributes[ 'attachments' ] ) > 0:
            url = submission.attachments[ 0 ][ 'url' ]
            # download the submitted file
            # print( "Downloading: ", url )
            response = requests.get( url, headers=make_request_header() )
            content = response.content

            # determine the appropriate handler to use
            filename = response.headers['content-disposition'].split('filename=')[1]
            filename = filename.replace('"', '')
            handler = chooseHandler(filename)

            # open the file and extract text
            return handler( content )

        else:
            # NB., if a student never submitted (workflow_state = 'unsubmitted'),
            # then they will have an entry which lacks a body key
            return None



if __name__ == '__main__':
    pass



"""
Created by adam on 1/29/20
"""
from unittest import TestCase

from CanvasHacks.GradingMethods.nonempty import receives_credit, CreditForNonEmptyOLD

__author__ = 'adam'

from TestingBase import TestingBase

if __name__ == '__main__':
    pass


class TestCreditForNonEmpty(TestingBase):

    def setUp(self) -> None:
        self.config_for_test()
        self.obj = CreditForNonEmptyOLD()

    def test_grade_credit( self ):
        txt = "The fat wiffle hound wiffled loudly"
        self.assertEqual( 100, self.obj.grade( txt ), "Defaults -- greater than minimum word count" )

    def test_grade_no_credit( self ):
        txt = ""
        self.assertEqual( None, self.obj.grade( txt ), "Defaults -- no credit" )


class Test_receives_credit( TestCase ):
    def test_happy_path_w_defaults( self ):
        txt = "The fat wiffle hound wiffled loudly"
        self.assertTrue(receives_credit(txt), "Defaults -- greater than minimum word count")

        txt = "hello"
        self.assertFalse(receives_credit(txt), "Defaults -- Less than minimum word count")

    def test_excluding_stopwords( self ):
        txt = "The fat wiffle hound wiffled loudly"
        self.assertTrue(receives_credit(txt, count_stopwords=False), "Excluding stopwords; still above minimum")

        txt = "The a dog"
        self.assertFalse(receives_credit(txt, count_stopwords=False), "Excluding stopwords pushes below minimum")


"""
Created by adam on 3/16/20
"""
__author__ = 'adam'

from CanvasHacks import environment
from CanvasHacks.DAOs.sqlite_dao import SqliteDAO
from CanvasHacks.Definitions.base import BlockableActivity
# todo Switch to quiz_new when ready
from CanvasHacks.GradingHandlers.factories import GradingHandlerFactory
from CanvasHacks.GradingHandlers.quiz import QuizGrader
from CanvasHacks.Models.model import StoreMixin
from CanvasHacks.Repositories.factories import WorkRepositoryLoaderFactory
from CanvasHacks.Repositories.reviewer_associations import AssociationRepository
from CanvasHacks.Repositories.submissions import QuizSubmissionRepository


class GradeQuiz( StoreMixin ):
    """
    Executable which grades and uploads scores for all selected
    assignments

    Taken from the code in the Quiz tools notebook which was created
    in CAN-44

    Created: CAN-44 / CAN-60
    """

    def __init__( self, activity=None, rest_timeout=5, no_late_penalty=True, upload_grades=True, **kwargs ):
        self.upload_grades = upload_grades
        self.wait = rest_timeout
        self.no_late_penalty = no_late_penalty
        if activity is None:
            # Use what's stored in environment
            selected_activities = environment.CONFIG.get_assignment_ids()
            # todo update to allow multiple selected activities
            activity_id = selected_activities[0]
            activity = environment.CONFIG.unit.get_activity_by_id(activity_id)
            # \
            # [ c for c in environment.CONFIG.unit.components if c.id == environment.CONFIG.get_assignment_ids.assignments[ 0 ][ 0 ] ][
            #     0 ]
        self.activity = activity

        if self.no_late_penalty:
            self.activity.due_at = self.activity.lock_at

        self.graded = [ ]

        self.handle_kwargs( **kwargs )

    def _initialize( self ):
        """
        Handle instantiating and loading repositories
        :return:
        """
        self.workRepo = WorkRepositoryLoaderFactory.make( course=environment.CONFIG.course,
                                                          activity=self.activity,
                                                          rest_timeout=self.wait )

        try:
            self.quiz = environment.CONFIG.course.get_quiz( self.activity.quiz_id )
        except AttributeError:
            # todo dev hope this doesn't cause problems!
            self.quiz = environment.CONFIG.course.get_quiz( self.activity.id )

        self.subRepo = QuizSubmissionRepository( self.quiz )

        # If this is a quiz type assignment, we need to get the quiz submission objects
        # not the regular submission object so we can use them for uploading
        self.qsubs = [ s for s in self.quiz.get_submissions() ]

        # Filter previously graded
        self.subRepo.data = [ s for s in self.subRepo.data if s.workflow_state != 'complete' ]
        self.workRepo.data = self.workRepo.data[ self.workRepo.data.workflow_state != 'complete' ].copy( deep=True )

        self.workRepo.data.reset_index( inplace=True )

        # We will need the association repo if the activity can be
        # blocked or graded by another student
        self.association_repo = None
        if isinstance( self.activity, BlockableActivity ):
            dao = SqliteDAO()
            self.association_repo = AssociationRepository( dao, self.activity )

    def run( self, **kwargs ):
        self.handle_kwargs( **kwargs )
        # Load data
        self._initialize()
        # Let the grader do its job
        grader = GradingHandlerFactory.make( activity=self.activity,
                                             work_repo=self.workRepo,
                                             submission_repo=self.subRepo,
                                             association_repo=self.association_repo )

        # grader = QuizGrader( work_repo=self.workRepo,
        #                      submission_repo=self.subRepo,
        #                      association_repo=self.association_repo )
        g = grader.grade( on_empty=0 )
        self.graded += g
        print( "Graded: ", len( self.graded ) )

        if self.upload_grades:
            self._upload_step()

    def get_submission_object( self, student_id, attempt ):
        return [ d for d in self.qsubs if d.user_id == student_id and d.attempt == attempt ]

    def _upload_step( self ):
        self.uploaded = 0
        # Upload grades
        for g in self.graded:
            sub = self.get_submission_object( g[ 'student_id' ], g[ 'attempt' ] )[ 0 ]
            sub.update_score_and_comments( quiz_submissions=g[ 'data' ][ 'quiz_submissions' ] )
            self.uploaded += 1


        self._log_uploaded()

    def _log_uploaded( self ):
        """
        Prints or writes to file what happened during upload
        :return:
        """
        stem = "Grades uploaded for {} students "
        print(stem.format(self.uploaded))



if __name__ == '__main__':
    # todo parse command line args into environment

    step = GradeQuiz()
    # step.run()



"""
Created by adam on 3/14/20
"""
__author__ = 'adam'

if __name__ == '__main__':
    pass

from CanvasHacks.Messaging.nagging import ReviewNonSubmittersMessaging


class SkaaReviewNagger:
    def __init__( self, overview_repo, send=True ):
        self.send = send
        self.overview_repo = overview_repo
        self.studentRepo = self.overview_repo.studentRepo


    @property
    def recipients( self ):
        """
        Returns list of (canvas id, name)
        :return:
        """
        return [ (cid, self.studentRepo.get_student_first_name( cid )) for cid in
                 self.overview_repo.non_reviewed.reviewed_by_id.tolist() ]

    def run( self ):
        self.unit = self.overview_repo.unit

        self.messenger = ReviewNonSubmittersMessaging( self.unit, send=self.send )

        print( "Going to nag {} students to turn in SKAA review".format( len( self.recipients ) ) )

        for cid, name in self.recipients:
            self.messenger.send_message_to_student( cid, name )



"""
Created by adam on 3/10/20
"""
__author__ = 'adam'

import pandas as pd
from canvasapi.quiz import QuizSubmission
from canvasapi.submission import Submission

if __name__ == '__main__':
    pass


class UncheckableDataProvidedError(Exception):
    """A filtration class has been provided with something
    it can't check"""
    pass


class IReadinessFilter:

    def is_ready( self, item ):
        raise NotImplementedError

    def _is_blocked_by_workflow_state( self, state ):
        """Given a value in a workflow_state field, returns
        what should do about it
        :return: Boolean or None if cannot determine
        """
        states = {
            # Workflow states found in attribute of canvas object
            'untaken': True,
            'pending_review': False,
            'complete': True,
            'settings_only': True,
            'preview': True
        }

        return states.get( state, None )

class ReadinessFilterFactory:

    @staticmethod
    def make( item ):
        """
        Determine what filter to return
        :param item:
        :return:
        """
        if isinstance(item, pd.Series):
            return FrameRowReadinessFilter()

        if isinstance( item, Submission ):
            return SubmissionReadinessFilter()

        if isinstance( item, QuizSubmission ):
            return QuizSubmissionReadinessFilter()

        raise UncheckableDataProvidedError(item)


class SubmissionReadinessFilter(IReadinessFilter):
    """
    Checks whether a Canvas api submission object
    represents something which is ready to be acted upon
    """

    def is_ready( self, item ):
        """
        Identifies assignments which have been previously graded
        :return: Boolean
        """
        try:
            # If the item is graded incomplete/complete
            # having the grade 'complete' means that
            # the item has been handled
            if item.grade == 'complete':
                return False
            # Other varients of grade should go here
            # or can we just check for none

            return not self._is_blocked_by_workflow_state( item.workflow_state )

        except AttributeError:
            # can't tell what to do
            raise UncheckableDataProvidedError(item)


class FrameRowReadinessFilter(IReadinessFilter):

    def is_ready( self, item, question_columns=None ):
        """
        If we've been given a data frame row,
        it will be a Series. Check whether it
        shows the person is unready
        :param item:
        :return:
        """
        if 'workflow_state' in item.index:
            if item.workflow_state == 'unsubmitted':
                return False
            # If the value is 'submitted' it could still be unready
            # so we continue

        # Any other column values based tests should go here.

        try:
            # Returning Boolean because assume that all other
            # possible conditions have been checked above

            if question_columns is not None:
                # If all the question columns are empty
                # it should be left alone
                for c in question_columns:
                    if len( item[ 0 ] ) > 0:
                        # Something contains at least one character
                        return True
                # All columns are empty
                return False

        except AttributeError:
            raise UncheckableDataProvidedError(item)


class QuizSubmissionReadinessFilter(IReadinessFilter):

    def is_ready( self, item ):
        return item.overdue_and_needs_submission



#
# class UnreadyFilterMixin:
#     """Tools for filtering out
#     previously graded and unsubmitted work from
#     any type of assignment.
#
#     These are for detecting based on server-side info; does not check my app. Detecting
#     whether they have been, e.g., assigned a reviewer is handled
#     elsewhere.
#
#     Private methods will return either True, False, or None
#     They will return None if the method is unable to make a determination
#
#     Public methods will return only True or False
#     """
#
#     # def check( self, item ):
#
#     def _is_blocked_by_workflow_state( self, state ):
#         """Given a value in a workflow_state field, returns
#         what should do about it
#         :return: Boolean or None if cannot determine
#         """
#         states = {
#             # Workflow states found in reports
#             'submitted': False,
#             'unsubmitted': True,
#
#             # Workflow states found in attribute of canvas object
#             'untaken': True,
#             'pending_review': False,
#             'complete': True,
#             'settings_only': True,
#             'preview': True
#         }
#
#         return states.get( state, None )
#
#
#
#     def is_ready( self, item ):
#         """
#         Main method which handles all they type checks
#         behind the scenes, to simply return a boolean of whether
#         the piece of work is ready to be acted on
#         :param item:
#         :return: Boolean
#         """
#         if isinstance(item, pd.Series):
#             return self._frame_row_blocks(item)
#
#         if isinstance( item, Submission ):
#             return self._submission_blocks( item )
#
#
#         #
#         # r = self._is_graded( item )
#         # if r is True:
#         #     return False
#         # # continue
#         #
#         # r2 = self._is_unsubmitted( item )
#         # if r2 is True:
#         #     return False
#         #
#         # r3 = self._is_blocked_by_workflow_state( item )
#         # if r3 is True:
#         #     return False
#
#         return not self._is_graded( item ) and not self._is_unsubmitted( item )



"""
Created by adam on 2/24/20
"""
import json

from CanvasHacks import environment as env

__author__ = 'adam'

if __name__ == '__main__':
    pass


def save_to_log_folder( frame ):
    section = frame.iloc[ 0 ][ 'section' ]
    qid = frame.iloc[ 0 ][ 'quiz_id' ]
    fname = "%s/%s-%s-results.xlsx" % (env.LOG_FOLDER, section, qid)
    print( "Saving to ", fname )
    frame.to_excel( fname )


def save_json( grade_data, quiz_data_obj ):
    fpath = "%s/%s-%s-all-submissions.json" % (env.LOG_FOLDER, quiz_data_obj.course_id, quiz_data_obj.id)
    # save submissions
    with open( fpath, 'w' ) as fpp:
        json.dump( grade_data, fpp )


"""
Created by adam on 2/24/20
"""
__author__ = 'adam'

if __name__ == '__main__':
    pass

class NoNewSubmissions(Exception):
    """Raised when there is, ahem, nothing new
    in the data which has been retrieved
    """
    pass

class NoStudentWorkDataLoaded(Exception):
    """Raised to indicate that ALL attempts to acquire data
    have failed and no more will be attempted"""
    pass

class NoWorkDownloaded(Exception):
    """
    Raised to indicate that the download attempts have failed
    so a handler can determine the next steps
    """
    def __init__(self):
        self.message = "Could not download data."


class NoWorkFromFile(Exception):
    """
    Raised to indicate that the download attempts have failed
    so a handler can determine the next steps
    """
    def __init__(self):
        self.message = "Could not load data from file."



"""
Created by adam on 5/6/19
"""
from CanvasHacks.Models.QuizModels import StoredDataFileMixin, QuizDataMixin
from CanvasHacks.Models.student import Student
from CanvasHacks.Definitions.skaa import Review
from CanvasHacks.Processors.cleaners import TextCleaner
from CanvasHacks.Processors.quiz import process_work, remove_non_final_attempts
from CanvasHacks.Files.QuizReportFileTools import retrieve_quiz_data, save_downloaded_report
from CanvasHacks.Repositories.mixins import StudentWorkMixin, SelectableMixin, FrameStorageMixin
from CanvasHacks.Repositories.submissions import QuizSubmissionRepository
from CanvasHacks.Text.stats import WordCount
from CanvasHacks.Widgets.AssignmentSelection import make_selection_button

__author__ = 'adam'

import pandas as pd

from CanvasHacks.Messaging.skaa import make_prompt_and_response
from CanvasHacks.Repositories.interfaces import IContentRepository


# def detect_question_columns(columns):
#     """Return a list of columns which contain a colon,
#     those probably contain the question answers
#     """
#     return [c for c in columns if len(c.split(':')) > 1]


# test = [ 'submitted', 'attempt',"1785114: \nWhat is an example of persuasive advertising?", '1.0']
# assert(detect_question_columns(test) == [ "1785114: \nWhat is an example of persuasive advertising?"])
# Limit to just the final attempts


#     return frame[pd.notnull(frame['submission_id'])]


#     to_drop = [ ]
#     for d in droppable:
#         for c in columns:
#             if c[ :len( d ) ] == d:
#                 to_drop.append( c )
#     return to_drop


# test = [ 'name', 'id', 'sis_id', '1.0', '1.0.1', '1.0.2' ]
# assert (make_drop_list( test ) == [ '1.0', '1.0.1', '1.0.2' ])


class QuizRepository( IContentRepository, QuizDataMixin, StoredDataFileMixin, StudentWorkMixin, SelectableMixin, FrameStorageMixin ):
    """Manages the data for a quiz type unit"""

    def __init__( self, activity, course=None ):
        self.course = course
        self.activity = activity
        self.question_columns = [ ]

        # The cleaner class that will be called to
        # remove html and other messy stuff from student
        # work
        self.text_cleaner = TextCleaner()

        self.analyzer = WordCount()

    def process( self, work_frame, submissions ):
        self.submissions = submissions
        if not isinstance( submissions, pd.DataFrame ):
            submissions_frame = pd.DataFrame( submissions )
        else:
            submissions_frame = submissions

        # If we are loading from file the student_id may
        # already have been set
        # try:
        #     v = submissions_frame['student_id']
        # except KeyError:
        submissions_frame[ 'student_id' ] = submissions_frame.user_id
        self.data = process_work( work_frame, submissions_frame )
        remove_non_final_attempts( self.data )
        # finish setting up the dataframe
        self._cleanup_data()
        # Store the text column names
        self.set_question_columns( self.data )

    def _cleanup_data( self ):
        """
        Runs cleanup operations specific to this kind of data
        """
        # the name will be set as index from sorting
        # so we set to student id to make look ups easier
        self.data.set_index( 'student_id', inplace=True )
        # Remove unneeded columns
        # self.data = self.data[self.activity_inviting_to_complete.question_columns]

        # Remove html and other artifacts from student answers
        # DO NOT UNCOMMENT UNTIL CAN-59 HAS BEEN FULLY TESTED
        for c in self.question_columns:
            self.data[c] = self.data.apply(lambda x: self.text_cleaner.clean(x[c]), axis=1)


    def get_student_work( self, student_id ):
        try:
            return self.data.loc[ student_id ]
        except (ValueError, KeyError):
            # The student id may not be set as the index, depending
            # on the source of the data
            return self.data.set_index( 'student_id' ).loc[ student_id ]

    def get_formatted_work_by( self, student_id ):
        """Returns all review entries by the student, formatted for
        sending out for review or display"""
        work = self.get_student_work( student_id )
        # narrow down to just the relevant columns
        rs = [ { 'prompt': column_name, 'response': work[ column_name ] } for col_id, column_name in
               self.question_columns ]
        r = make_prompt_and_response( rs )
        return self._check_empty( r )

    def make_question_selection_buttons( self ):
        """Given a repository containing a dataframe and a
        list of names in question_names, this will allow to select
        which questions are used for things"""
        buttons = [ ]
        for q in self.question_names:
            b = make_selection_button( q, q, self.get_selections, self.select, self.deselect, '100%' )
            buttons.append( b )

    @property
    def points_per_question( self ):
        return self.quiz.points_possible / self.quiz.question_count

    @property
    def quiz( self ):
        """Returns the canvasapi.quiz.Quiz object associated
        with this repository.
        Automatically initializes it if not set
        """
        try:
            if self._quiz:
                pass
        except AttributeError:
            self._quiz = self.course.get_quiz( self.activity.quiz_id )
        return self._quiz

    @property
    def submitters( self ):
        """returns a list of student objects for whom work has been submitted"""
        return [ Student( s ) for s in self.student_ids ]

    @property
    def submitter_ids( self ):
        """Returns a list of canvas ids of students who have submitted the unit"""
        # try:
        return list( set( self.data.reset_index().student_id.tolist() ) )
        # except (ValueError, KeyError):


    def word_counts( self ):
        """
        Returns a dataframe with columns student_id, and all
        question columns. Question columns contain word counts
        :return:
        """
        d = []
        for i, row in self.data.iterrows():
            s = {'student_id': i }
            for c in self.question_columns:
                s[c] = self.analyzer.analyze(row[c])
            d.append(s)
        return pd.DataFrame(d)


class ReviewRepository( QuizRepository ):
    """Quiz repo specific to needs of reviews.
    Basically just differs in implementation of
    get_formatted_work and related methods
    """

    def __init__( self, activity, course=None ):
        self.course = course
        self.activity = activity
        self.question_columns = [ ]
        if course:
            self.questions = self.course.get_quiz( self.activity.quiz_id ).get_questions()

    #
    # def _set_question_types( self ):
    #     def __init__( self, activity_inviting_to_complete ):
    #         self.activity_inviting_to_complete = activity_inviting_to_complete
    #     self.question_columns = []

    # @property
    # def questions( self ):
    #     """Returns canvasapi questions for the activity_inviting_to_complete"""
    #     print('getting qs')
    #     return self.course.get_quiz(self.activity_inviting_to_complete.quiz_id).get_questions()

    def _fix_forgot_answers( self ):
        def r( v ):
            if v == 'They forgot to do this':
                return 'Forgot'
            return v

        for c in self.multiple_choice_names:
            self.data[ c ] = self.data.apply( lambda x: r( x[ c ] ), axis=1 )

    @property
    def essay_questions_names( self ):
        essay_questions = [ q.id for q in self.questions if q.question_type == 'essay_question' ]
        return [ c[ 1 ] for c in self.question_columns if int( c[ 0 ] ) in essay_questions ]

    @property
    def multiple_choice_names( self ):
        multiple_choice = [ q.id for q in self.questions if q.question_type == 'multiple_choice_question' ]
        return [ c[ 1 ] for c in self.question_columns if int( c[ 0 ] ) in multiple_choice ]

    @property
    def question_names( self ):
        return self.multiple_choice_names + self.essay_questions_names

    def get_formatted_work_by( self, student_id ):
        """Returns all entries by the indicated student, formatted for
        sending in a message or display"""

        def format_feedback( prompt, response ):
            return """
            ========
            Prompt: 
            {}
            
            Response: 
            {}
            =========
            """.format( prompt, response )

        # todo Add a check to make sure content is non empty. Raise an error if it is so other methods can decide what to do
        work = self.get_student_work( student_id )

        content = [ ]
        for c in self.question_names:
            content.append( format_feedback( c.split( ':' )[ 1 ], work[ c ] ) )

        content = "\n".join( content )
        return content


if __name__ == '__main__':
    pass


def make_quiz_repo( course, activity, save=True ):
    """Gets all student work data for the activity_inviting_to_complete that's part of the unit
    loads it into a QuizRepository or ReviewRepository and
    returns the repository.
    This is the main method called to get data
    """
    # Get quiz submission objects
    # repo = WorkRepositoryFactory.make( activity_inviting_to_complete, course )
    if isinstance(activity, Review):
        repo = ReviewRepository(activity, course)
    else:
        repo = QuizRepository(activity, course)

    # Download student work
    # This will work if the 'Create Report' button has been manually clicked
    student_work_frame = retrieve_quiz_data( repo.quiz )

    if save:
        # Want to have all the reports be formatted the same
        # regardless of whether we manually or programmatically
        # downloaded them. Thus we save before doing anything to them.
        save_downloaded_report( activity, student_work_frame )

    # Download submissions
    subRepo = QuizSubmissionRepository( repo.quiz )

    # Doing the combination with submissions after saving to avoid
    # mismatches of new and old data
    repo.process( student_work_frame, subRepo.frame )

    return repo


"""
Created by adam on 1/18/20
"""
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()

__author__ = 'adam'

if __name__ == '__main__':
    pass


class SqliteDAO( object ):
    """
    Makes a connection to sqlite database.
    [following is from import from twitter tools]
    Note that does not actually populate the database. That
    requires a call to: Base.metadata.create_all(SqliteConnection)
    """

    def __init__( self, db_filepath=None ):
        if db_filepath:
            self._make_file_engine( db_filepath )
        else:
            self._create_memory_engine()

        self._connect()

    def _connect( self ):
        """
        Creates a session on self.session
        :return:
        """
        self.session_factory = sessionmaker( bind=self.engine )
        self.session = self.session_factory()

    def _create_memory_engine( self ):
        """
        Creates an in-memory sqlite db engine
        """
        connection_string = 'sqlite:///:memory:'
        print( "creating connection: %s " % connection_string )
        self.engine = create_engine( connection_string, echo=False )

        Base.metadata.create_all( self.engine )
        # print( "creating connection: %s " % conn )
        # self.engine = create_engine( conn, echo=False )

    def initialize_db_file( self ):
        """
        Creates tables in the database
        :return:
        """
        Base.metadata.create_all( self.engine )


    def _make_file_engine( self, filepath ):
        connection_string = 'sqlite:///{}'.format( filepath )
        print( "creating connection: %s " % connection_string )
        self.engine = create_engine( connection_string, echo=False )



"""
Created by adam on 3/16/20
"""
__author__ = 'adam'

import re

import canvasapi

from CanvasHacks.Definitions.skaa import InitialWork, Review, MetaReview
from CanvasHacks.Definitions.other import TopicalAssignment, UnitEndSurvey
from CanvasHacks.Definitions.discussion import DiscussionForum, DiscussionReview

if __name__ == '__main__':
    pass


class Unit:
    """The main SKAA. This holds the definitions of all the consituent parts"""
    __name__ = 'Unit'

    def __init__( self, course, unit_number ):
        self.component_types = [ TopicalAssignment,
                                 InitialWork,
                                 Review,
                                 MetaReview,
                                 DiscussionForum,
                                 DiscussionReview,
                                 UnitEndSurvey
                                 ]
        self.components = [ ]

        self.course = course
        self.unit_number = unit_number
        if isinstance( course, canvasapi.course.Course ):
            # This check is here so can run tests without
            # needing a dummy Course object
            self._initialize()

    # def __repr__(self):
    #     """This way can still use unit in format statements """
    #     return self.unit_number

    def _initialize( self ):
        # Get all assignments for the course
        assignments = [ a for a in self.course.get_assignments() ]
        print( "{} assignments in course".format( len( assignments ) ) )
        # Parse out the assignments which have the unit number in their names
        unit_assignments = self.find_for_unit( self.unit_number, assignments )
        print( "{} assignments found for unit # {}".format( len( unit_assignments ), self.unit_number ) )
        self.find_components( unit_assignments )
        for c in self.components:
            # todo access_code_for_next_on probably not needed; created without looking at what already have
            # self._set_access_code_for_next( c )
            # Set the unit number for the assignment
            setattr( c, 'unit_number', self.unit_number )

    def find_components( self, unit_assignments ):
        # Parse components of unit
        for t in self.component_types:
            for a in unit_assignments:
                if t.is_activity_type( a.name ):
                    o = t( **a.attributes )
                    o.access_code = self._set_access_code( o )
                    self.components.append( o )

    def find_for_unit( self, unit_number, assignments ):
        """Given a list of unit names finds the one's
        relevant to this unit
        """
        rx = re.compile( r"\bunit {}\b".format( unit_number ) )
        try:
            return [ a for a in assignments if rx.search( a.name.strip().lower() ) ]
        except AttributeError:
            # Things like discussion forums have a title, not a name
            return [ a for a in assignments if rx.search( a.title.strip().lower() ) ]

    def _set_access_code_for_next( self, obj ):
        """Sets the access code students will need for the subsequent
        unit on the present unit as access_code_for_next.
        NB, this must be run after all the unit.components have been
        discovered and had their access codes set
        """
        # todo access_code_for_next_on probably not needed; created without looking at what already have

        try:
            if obj.access_code_for_next_on is not None:
                next_assign = self.get_by_class( obj.access_code_for_next_on )
                obj.access_code_for_next = next_assign.access_code
        except AttributeError:
            print( "No access code for subsequent unit set for {}".format( obj.name ) )

    def _set_access_code( self, obj ):
        """Some things will have an access code stored
        on them. Others have no access code. Some have
        the access code stored elsewhere.
        This sorts that out and sets the access code if possible.
        NB, this sets the code for the present unit, not the unit
        we will be notifying about
        """
        try:
            # first we try to set from self
            if obj.access_code is not None:
                return obj.access_code
            raise AttributeError
        except AttributeError:
            # check to see if we ahve a quiz id
            try:
                return self.course.get_quiz( obj.quiz_id ).access_code
            except AttributeError:
                print( "No access code for {}".format( obj.name ) )
                return None

    def get_by_class( self, component_class ):
        """
        Returns the unit component given an unistantiated
        class definition object
        :param component_class:
        :return:
        """
        for c in self.components:
            if isinstance( c, component_class ):
                return c

    def get_activity_by_id( self, activity_id ):
        for c in self.components:
            if c.id == activity_id:
                return c

    def get_discussion_by_topic_id( self, topic_id ):
        """
        Discussions are usually identified via the topic id
        thus this is a shortcut to getting the right one, if there
        is more than one associated with the unit
        :param topic_id:
        :return:
        """
        for c in self.components:
            try:
                if c.topic_id == topic_id:
                    return c
            except AttributeError:
                pass

    @property
    def topical_assignment( self ):
        for c in self.components:
            if isinstance( c, TopicalAssignment ):
                return c

    @property
    def initial_work( self ):
        for c in self.components:
            if isinstance( c, InitialWork ):
                return c

    @property
    def metareview( self ):
        for c in self.components:
            if isinstance( c, MetaReview ):
                return c

    @property
    def review( self ):
        for c in self.components:
            if isinstance( c, Review ):
                return c

    @property
    def discussion_forum( self ):
        for c in self.components:
            if isinstance( c, DiscussionForum ):
                return c

    @property
    def discussion_review( self ):
        for c in self.components:
            if isinstance( c, DiscussionReview ):
                return c

    @property
    def unit_end_survey( self ):
        for c in self.components:
            if isinstance( c, UnitEndSurvey ):
                return c


"""
Created by adam on 2/23/20
"""

__author__ = 'adam'

from CanvasHacks.Models.student import get_first_name
from CanvasHacks.Repositories.status import FeedbackStatusRepository

if __name__ == '__main__':
    pass

"""
Created by adam on 2/22/20
"""
from tests.TestingBase import TestingBase
from unittest.mock import MagicMock, patch, create_autospec

from faker import Faker

from CanvasHacks.Messaging.templates import METAREVIEW_CONTENT_TEMPLATE
from CanvasHacks.Repositories.students import StudentRepository

from tests.factories.ModelFactories import student_factory
from tests.factories.PeerReviewedFactories import unit_factory
from tests.factories.RepositoryMocks import ContentRepositoryMock

from CanvasHacks.Messaging.skaa import FeedbackFromMetareviewMessenger

fake = Faker()

__author__ = 'adam'

if __name__ == '__main__':
    pass


class TestFeedbackFromMetareviewMessenger( TestingBase ):

    def setUp( self ):
        self.config_for_test()
        self.unit = unit_factory()
        # self.activity_data = activity_data_factory()
        self.activity = self.unit.metareview #InitialWork( **self.activity_data )

        self.author = student_factory()
        # student receiving the message (we are sending the feedback
        # created by the author of the content assignment)
        self.reviewer = student_factory()

        # This would be metareview feedback on the review
        self.work = fake.text()

        self.studentRepo = StudentRepository()
        self.studentRepo.get_student = MagicMock( return_value=self.reviewer )
        self.contentRepo = ContentRepositoryMock()
        self.contentRepo.get_formatted_work_by = MagicMock( return_value=self.work )

        self.review_assign = MagicMock( assessor_id=self.reviewer.id, assessee_id=self.author.id )
        self.statusRepo = create_autospec(FeedbackStatusRepository)

    def test_prepare_message( self ):
        obj = FeedbackFromMetareviewMessenger( self.unit, self.studentRepo, self.contentRepo, self.statusRepo )

        # call
        message_data = obj.prepare_message( self.review_assign )

        # check
        self.assertEqual( obj.message_template, METAREVIEW_CONTENT_TEMPLATE, "Working off expected template" )
        self.assertEqual( message_data[ 'student_id' ], self.reviewer.id, "Message is going to reviewer (it contains the feedback from the original author)" )
        self.assertEqual( message_data[ 'subject' ], FeedbackFromMetareviewMessenger.subject.format(self.unit.unit_number), "Sent with expected subject line (This is different from the tests of its siblings" )

        self.assertTrue( len( message_data[ 'body' ] ) > 0 )

        d = {
            'intro': FeedbackFromMetareviewMessenger.intro,
            # the person who did the review is getting feedback from the author
            'name': get_first_name( self.reviewer ),

            # Formatted work for sending
            'responses': self.work,

            # Add any materials from me
            'other': "",
        }
        expected_content = METAREVIEW_CONTENT_TEMPLATE.format( **d )

        self.assertEqual( expected_content, message_data[ 'body' ], "Expected message body" )

        # Super important: makes sure going to right person
        # This is for the metareview, so the receipient should be the REVIEWER
        self.studentRepo.get_student.assert_called_with(self.reviewer.id )

    @patch( 'CanvasHacks.Messaging.SendTools.ConversationMessageSender.send' )
    def test_notify( self, sendMock ):
        sendMock.return_value = 'taco'
        self.obj = FeedbackFromMetareviewMessenger( self.unit, self.studentRepo, self.contentRepo, self.statusRepo )

        # Call
        result = self.obj.notify( [ self.review_assign ], send=True )

        # Check
        # Intermediate things were called
        self.assertTrue(self.contentRepo.get_formatted_work_by.called, "Content repo method called")
        self.assertTrue(self.studentRepo.get_student.called, "Student repo method called")
        # Returned expected stuff
        self.assertTrue( len( result ) == 1, "Something was returned" )

        # Check that the sender was given the expected content
        sendMock.assert_called()
        kwargs = sendMock.call_args[1 ]
        self.assertEqual( kwargs['student_id'], self.reviewer.id, "Sent to reviewer" )
        self.assertEqual(kwargs['subject'], FeedbackFromMetareviewMessenger.subject.format(self.unit.unit_number), "Sent with expected subject line (This is different from the tests of its siblings" )
        d = {
            'intro': FeedbackFromMetareviewMessenger.intro,
            # the person who did the review is getting feedback from the author
            'name': get_first_name( self.reviewer ),

            # Formatted work for sending
            'responses': self.work,

            # Add any materials from me
            'other': "",
        }
        # d = self.obj._make_template_input( self.work, None, self.reviewer )
        b = METAREVIEW_CONTENT_TEMPLATE.format(**d)
        self.assertEqual(kwargs['body'], b, "Sent with expected body")

        # Super important: makes sure going to right person
        # This is for the metareview, so the receipient should be the REVIEWER
        self.studentRepo.get_student.assert_called_with(self.reviewer.id )

        # Check that status repo called
        self.statusRepo.record.assert_called()
        self.statusRepo.record.assert_called_with( self.reviewer.id )



"""
Simulating data from the canvas api

Created by adam on 12/24/19
"""

__author__ = 'adam'

from faker import Faker

fake = Faker()

import random

import pandas as pd


if __name__ == '__main__':
    pass


def submission_result_factory(activity, student, on_time=True):
    """"""
    return {
        # The submission's unit id
        "assignment_id": activity.id,
        # The submission's unit (see the assignments API) (optional)
        "unit": None,
        # The submission's course (see the course API) (optional)
        "course": None,
        # This is the submission attempt number.
        "attempt": 1,
        # The content of the submission, if it was submitted directly in a text field.
        "body": fake.paragraph(),
        # The grade for the submission, translated into the unit grading scheme
        # (so a letter grade, for example).
        "grade": "A-",
        # A boolean flag which is false if the student has re-submitted since the
        # submission was last graded.
        "grade_matches_current_submission": True,
        # URL to the submission. This will require the user to log in.
        "html_url": "http:#example.com/courses/255/assignments/543/submissions/134",
        # URL to the submission preview. This will require the user to log in.
        "preview_url": "http:#example.com/courses/255/assignments/543/submissions/134?preview=1",
        # The raw score
        "score": random.randint(1,1000),
        # Associated comments for a submission (optional)
        "submission_comments": None,
        # The types of submission ex:
        # ('online_text_entry'|'online_url'|'online_upload'|'media_recording')
        "submission_type": "online_text_entry",
        # The timestamp when the unit was submitted
        "submitted_at": activity.due_at - pd.Timedelta("1 Day") if on_time else activity.due_at + pd.Timedelta("1 Day") ,
        # The URL of the submission (for 'online_url' submissions).
        "url": None,
        # The id of the user who created the submission
        "user_id": student.student_id,
        # The id of the user who graded the submission. This will be None for
        # submissions that haven't been graded yet. It will be a positive number if a
        # real user has graded the submission and a negative number if the submission
        # was graded by a process (e.g. Quiz autograder and autograding LTI tools).
        # Specifically autograded quizzes set grader_id to the negative of the quiz id.
        # Submissions autograded by LTI tools set grader_id to the negative of the tool
        # id.
        "grader_id": 86,
        "graded_at": "2012-01-02T03:05:34Z",
        # The submissions user (see user API) (optional)
        "user": None,
        # Whether the submission was made after the applicable due date
        "late": False,
        # Whether the unit is visible to the user who submitted the unit.
        # Submissions where `assignment_visible` is false no longer count towards the
        # student's grade and the unit can no longer be accessed by the student.
        # `assignment_visible` becomes false for submissions that do not have a grade
        # and whose unit is no longer assigned to the student's section.
        "assignment_visible": True,
        # Whether the unit is excused.  Excused assignments have no impact on a
        # user's grade.
        "excused": True,
        # Whether the unit is missing.
        "missing": True,
        # The status of the submission in relation to the late policy. Can be late,
        # missing, none, or None.
        "late_policy_status": "missing",
        # The amount of points automatically deducted from the score by the
        # missing/late policy for a late or missing unit.
        "points_deducted": 12.3,
        # The amount of time, in seconds, that an submission is late by.
        "seconds_late": 300,
        # The current state of the submission
        "workflow_state": "submitted",
        # Extra submission attempts allowed for the given user and unit.
        "extra_attempts": 10,
        # A unique short ID identifying this submission without reference to the owning
        # user. Only included if the caller has administrator access for the current
        # account.
        "anonymous_id": "acJ4Q",
        # The date this submission was posted to the student, or nil if it has not been
        # posted.
        "posted_at": "2020-01-02T11:10:30Z"
    }


def peer_review_result_factory( submitter, reviewer, completed=True ):
    return {
        # The assessors user id
        "assessor_id": reviewer.student_id,
        # The id for the asset associated with this Peer Review
        "asset_id": 13,
        # The type of the asset
        "asset_type": "Submission",
        # The id of the Peer Review
        "id": 1,
        # The user id for the owner of the asset
        "user_id": submitter.student_id,
        # The state of the Peer Review, either 'assigned' or 'completed'
        "workflow_state": 'completed' if completed else "assigned",
        # the User object for the owner of the asset if the user include parameter is
        # provided (see user API) (optional)
        "user": "User",
        # The User object for the assessor if the user include parameter is provided
        # (see user API) (optional)
        "assessor": "User",
        # The submission comments associated with this Peer Review if the
        # submission_comment include parameter is provided (see submissions API)
        # (optional)
        "submission_comments": submission_comment_result_factory(reviewer)
    }


def submission_comment_result_factory( commentor ):
    """See https://canvas.instructure.com/doc/api/submissions.html#method.submissions_api.update"""
    return {
        "id": 37,
        "author_id": commentor.student_id,
        "author_name": commentor.name,
        # Abbreviated user object UserDisplay (see users API).
        "author": "{}",
        "comment": fake.paragraph(),
        "created_at": "2012-01-01T01:00:00Z",
        "edited_at": "2012-01-02T01:00:00Z",
        "media_comment": None
    }


"""
Created by adam on 2/28/20
"""
__author__ = 'adam'

from CanvasHacks.Errors.messaging import MessageDataCreationError
from CanvasHacks.Messaging.base import SkaaMessenger
from CanvasHacks.Messaging.templates import DISCUSSION_REVIEW_NOTICE_TEMPLATE, DISCUSSION_REVIEW_FEEDBACK_TEMPLATE
from CanvasHacks.Models.student import get_first_name
from CanvasHacks.Definitions.unit import Unit
from CanvasHacks.Repositories.status import StatusRepository

if __name__ == '__main__':
    pass


class DiscussionReviewInvitationMessenger( SkaaMessenger ):
    """Handles invitation to complete discussion review"""
    message_template = DISCUSSION_REVIEW_NOTICE_TEMPLATE

    def __init__( self, unit: Unit, student_repository, content_repository,
                  status_repositories: StatusRepository ):
        self.activity_inviting_to_complete = unit.discussion_review

        super().__init__( unit, student_repository, content_repository, status_repositories )

        # self.subject = "Discussion forum posts for you to review"
        # self.intro = "Here are some posts by another student for you to review. "

    def prepare_message( self, review_assignment, other=None ):
        """This looks up the appropriate data for a review
        unit and returns what will be the message body
        """
        try:
            # We are going to send the posts to the reviewer
            receiving_student = self.student_repository.get_student( review_assignment.assessor_id )

            # The assessee did the work (i.e., posts) that we want to send
            # to the assessor
            content = self.content_repository.get_formatted_work_by( review_assignment.assessee_id )

            return self._make_message_data( receiving_student, content, other=None )

        except Exception as e:
            # todo exception handling
            print( e )
            raise MessageDataCreationError( review_assignment )


class FeedbackFromDiscussionReviewMessenger( SkaaMessenger ):
    message_template = DISCUSSION_REVIEW_FEEDBACK_TEMPLATE
    email_subject_templ = "Feedback on your Unit {} discussion forum posts"
    intro = "Here is the feedback from another student on your discussion forum posts. "

    def __init__( self, unit: Unit, student_repository, content_repository,
                  status_repositories: StatusRepository ):

        # We will use the subject and intro
        # stored on this class rather than deferring to the activity
        self.activity_inviting_to_complete = None

        self.email_subject = self.email_subject_templ.format(unit.unit_number)

        super().__init__( unit, student_repository, content_repository, status_repositories )

    def prepare_message( self, review_assignment, other=None ):
        """This looks up the appropriate data for a review
        unit and returns what will be the message body
        """
        try:
            # todo

            # We are going to send the posts to the author of the posts
            receiving_student = self.student_repository.get_student( review_assignment.assessee_id )

            # We want the content produced by the reviewer
            content = self.content_repository.get_formatted_work_by( review_assignment.assessor_id )

            d = {
                'intro': self.intro,

                'name': get_first_name( receiving_student ),

                # Formatted work for sending
                'responses': content,

                # Add any materials from me
                'other': "",
            }

            body = self.message_template.format( **d )

            # can't use the usual self.make_message_data because won't have all the expected fields
            return {
                'student_id': receiving_student.id,
                'subject': self.email_subject,
                'body': body
            }

        except Exception as e:
            # todo exception handling
            print( e )
            raise MessageDataCreationError( review_assignment )